@@ -40,7 +40,8 @@ class ReviveManager {
   private final int batchSize;
   ShuffleClientImpl shuffleClient;
   private final ScheduledExecutorService batchReviveRequestScheduler =
-      ThreadUtils.newDaemonSingleThreadScheduledExecutor("batch-revive-scheduler");
+      ThreadUtils.newDaemonSingleThreadScheduledExecutor(
+          "celeborn-client-lifecycle-manager-batch-revive-scheduler");
 
   public ReviveManager(ShuffleClientImpl shuffleClient, CelebornConf conf) {
     this.shuffleClient = shuffleClient;
@@ -80,7 +81,6 @@ class ReviveManager {
                 } else {
                   filteredRequests.add(req);
                   mapIds.add(req.mapId);
-                  PartitionLocation loc = req.loc;
                   if (!requestsToSend.containsKey(req.partitionId)
                       || requestsToSend.get(req.partitionId).epoch < req.epoch) {
                     requestsToSend.put(req.partitionId, req);
