@@ -61,6 +61,16 @@ public abstract class ShuffleClient {
       int port,
       CelebornConf conf,
       UserIdentifier userIdentifier) {
+    return ShuffleClient.get(appUniqueId, driverHost, port, conf, userIdentifier, null);
+  }
+
+  public static ShuffleClient get(
+      String appUniqueId,
+      String driverHost,
+      int port,
+      CelebornConf conf,
+      UserIdentifier userIdentifier,
+      byte[] extension) {
     if (null == _instance || !initialized) {
       synchronized (ShuffleClient.class) {
         if (null == _instance) {
@@ -72,11 +82,13 @@ public abstract class ShuffleClient {
           // when communicating with LifecycleManager, it will cause a NullPointerException.
           _instance = new ShuffleClientImpl(appUniqueId, conf, userIdentifier);
           _instance.setupLifecycleManagerRef(driverHost, port);
+          _instance.setExtension(extension);
           initialized = true;
         } else if (!initialized) {
           _instance.shutdown();
           _instance = new ShuffleClientImpl(appUniqueId, conf, userIdentifier);
           _instance.setupLifecycleManagerRef(driverHost, port);
+          _instance.setExtension(extension);
           initialized = true;
         }
       }
@@ -122,7 +134,28 @@ public abstract class ShuffleClient {
 
   public abstract void setupLifecycleManagerRef(RpcEndpointRef endpointRef);
 
-  // Write data to a specific reduce partition
+  /**
+   * @param extension Extension for shuffle client, it's a byte array. Used in derived shuffle
+   *     client implementation.
+   */
+  public abstract void setExtension(byte[] extension);
+
+  /**
+   * Write data to a specific reduce partition
+   *
+   * @param shuffleId the unique shuffle id of the application
+   * @param mapId the map id of the shuffle
+   * @param attemptId the attempt id of the map task, i.e. speculative task or task rerun for Apache
+   *     Spark
+   * @param partitionId the partition id the data belongs to
+   * @param data byte array containing data to be pushed
+   * @param offset start position of data to be pushed
+   * @param length length of data to be pushed
+   * @param numMappers the number map tasks in the shuffle
+   * @param numPartitions the number of partitions in the shuffle
+   * @return bytes pushed
+   * @throws IOException
+   */
   public abstract int pushData(
       int shuffleId,
       int mapId,
@@ -165,6 +198,18 @@ public abstract class ShuffleClient {
 
   // Reduce side read partition which is deduplicated by mapperId+mapperAttemptNum+batchId, batchId
   // is a self-incrementing variable hidden in the implementation when sending data.
+  /**
+   * @param shuffleId the unique shuffle id of the application
+   * @param partitionId the partition id to read from
+   * @param attemptNumber the attempt id of reduce task, can be safely set to any value
+   * @param startMapIndex the index of start map index of interested map range, set to 0 if you want
+   *     to read all partition data
+   * @param endMapIndex the index of end map index of interested map range, set to
+   *     `Integer.MAX_VALUE` if you want to read all partition data
+   * @param metricsCallback callback to report metrics
+   * @return
+   * @throws IOException
+   */
   public abstract CelebornInputStream readPartition(
       int shuffleId,
       int partitionId,
@@ -185,4 +230,13 @@ public abstract class ShuffleClient {
       int shuffleId, int numMappers, int numPartitions);
 
   public abstract PushState getPushState(String mapKey);
+
+  public abstract int getShuffleId(int appShuffleId, String appShuffleIdentifier, boolean isWriter);
+
+  /**
+   * report shuffle data fetch failure to LifecycleManager for special handling, eg, shuffle status
+   * cleanup for spark app. It must be a sync call and make sure the cleanup is done, otherwise,
+   * incorrect shuffle data can be fetched in re-run tasks
+   */
+  public abstract boolean reportShuffleFetchFailure(int appShuffleId, int shuffleId);
 }
