@@ -91,8 +91,22 @@ public abstract class CelebornInputStream extends InputStream {
         public int read(byte[] b, int off, int len) throws IOException {
           return -1;
         }
+
+        @Override
+        public int totalPartitionsToRead() {
+          return 0;
+        }
+
+        @Override
+        public int partitionsRead() {
+          return 0;
+        }
       };
 
+  public abstract int totalPartitionsToRead();
+
+  public abstract int partitionsRead();
+
   private static final class CelebornInputStreamImpl extends CelebornInputStream {
     private static final Random RAND = new Random();
 
@@ -168,15 +182,15 @@ public abstract class CelebornInputStream extends InputStream {
       this.fetchExcludedWorkerExpireTimeout = conf.clientFetchExcludedWorkerExpireTimeout();
       this.fetchExcludedWorkers = fetchExcludedWorkers;
 
-      int blockSize = conf.clientPushBufferMaxSize();
+      int bufferSize = conf.clientFetchBufferSize();
       if (shuffleCompressionEnabled) {
         int headerLen = Decompressor.getCompressionHeaderLength(conf);
-        blockSize += headerLen;
-        compressedBuf = new byte[blockSize];
+        bufferSize += headerLen;
+        compressedBuf = new byte[bufferSize];
 
         decompressor = Decompressor.getDecompressor(conf);
       }
-      rawDataBuf = new byte[blockSize];
+      rawDataBuf = new byte[bufferSize];
 
       if (conf.clientPushReplicateEnabled()) {
         fetchChunkMaxRetry = conf.clientFetchMaxRetriesForEachReplica() * 2;
@@ -579,5 +593,15 @@ public abstract class CelebornInputStream extends InputStream {
 
       return hasData;
     }
+
+    @Override
+    public int totalPartitionsToRead() {
+      return locations.length;
+    }
+
+    @Override
+    public int partitionsRead() {
+      return fileIndex;
+    }
   }
 }
