@@ -62,6 +62,9 @@ public class DummyShuffleClient extends ShuffleClient {
   @Override
   public void setupLifecycleManagerRef(RpcEndpointRef endpointRef) {}
 
+  @Override
+  public void setExtension(byte[] extension) {}
+
   @Override
   public int pushData(
       int shuffleId,
@@ -153,6 +156,16 @@ public class DummyShuffleClient extends ShuffleClient {
     return new PushState(conf);
   }
 
+  @Override
+  public int getShuffleId(int appShuffleId, String appShuffleIdentifier, boolean isWriter) {
+    return appShuffleId;
+  }
+
+  @Override
+  public boolean reportShuffleFetchFailure(int appShuffleId, int shuffleId) {
+    return true;
+  }
+
   public void initReducePartitionMap(int shuffleId, int numPartitions, int workerNum) {
     ConcurrentHashMap<Integer, PartitionLocation> map = JavaUtils.newConcurrentHashMap();
     String host = "host";
