@@ -18,7 +18,7 @@
 package org.apache.celeborn.common.client;
 
 import java.io.IOException;
-import java.util.UUID;
+import java.util.*;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
@@ -52,7 +52,7 @@ public class MasterClient {
   private static final Logger LOG = LoggerFactory.getLogger(MasterClient.class);
 
   private final RpcEnv rpcEnv;
-  private final String[] masterEndpoints;
+  private final List<String> masterEndpoints;
   private final int maxRetries;
 
   private final RpcTimeout rpcTimeout;
@@ -62,11 +62,13 @@ public class MasterClient {
 
   public MasterClient(RpcEnv rpcEnv, CelebornConf conf) {
     this.rpcEnv = rpcEnv;
-    this.masterEndpoints = conf.masterEndpoints();
-    this.maxRetries = Math.max(masterEndpoints.length, conf.masterClientMaxRetries());
+    this.masterEndpoints = Arrays.asList(conf.masterEndpoints());
+    Collections.shuffle(this.masterEndpoints);
+    this.maxRetries = Math.max(masterEndpoints.size(), conf.masterClientMaxRetries());
     this.rpcTimeout = conf.masterClientRpcAskTimeout();
     this.rpcEndpointRef = new AtomicReference<>();
-    this.oneWayMessageSender = ThreadUtils.newDaemonSingleThreadExecutor("One-Way-Message-Sender");
+    this.oneWayMessageSender =
+        ThreadUtils.newDaemonSingleThreadExecutor("celeborn-one-way-message-sender");
   }
 
   private static final String SPLITTER = "#";
@@ -168,7 +170,6 @@ public class MasterClient {
     // It will always throw celeborn exception , so we need to get the cause
     // 'CelebornException: Exception thrown in awaitResult'
     if (e.getCause() instanceof MasterNotLeaderException) {
-      // 0.4 and above Master returns org.apache.celeborn.common.client.MasterNotLeaderException
       MasterNotLeaderException exception = (MasterNotLeaderException) e.getCause();
       String leaderAddr = exception.getSuggestedLeaderAddress();
       if (!leaderAddr.equals(MasterNotLeaderException.LEADER_NOT_PRESENTED)) {
@@ -177,19 +178,6 @@ public class MasterClient {
         LOG.warn("Master leader is not present currently, please check masters' status!");
       }
       return true;
-    } else if (e.getCause()
-        instanceof org.apache.celeborn.common.haclient.MasterNotLeaderException) {
-      // 0.3 Master returns org.apache.celeborn.common.haclient.MasterNotLeaderException
-      org.apache.celeborn.common.haclient.MasterNotLeaderException exception =
-          (org.apache.celeborn.common.haclient.MasterNotLeaderException) e.getCause();
-      String leaderAddr = exception.getSuggestedLeaderAddress();
-      if (!leaderAddr.equals(
-          org.apache.celeborn.common.haclient.MasterNotLeaderException.LEADER_NOT_PRESENTED)) {
-        setRpcEndpointRef(leaderAddr);
-      } else {
-        LOG.warn("Master leader is not present currently, please check masters' status!");
-      }
-      return true;
     } else if (e.getCause() instanceof IOException) {
       resetRpcEndpointRef(oldRef);
       return true;
@@ -238,9 +226,9 @@ public class MasterClient {
     if (endpointRef == null) {
       int index = currentIndex.get();
       do {
-        RpcEndpointRef tempEndpointRef = setupEndpointRef(masterEndpoints[index]);
+        RpcEndpointRef tempEndpointRef = setupEndpointRef(masterEndpoints.get(index));
         if (rpcEndpointRef.compareAndSet(null, tempEndpointRef)) {
-          index = (index + 1) % masterEndpoints.length;
+          index = (index + 1) % masterEndpoints.size();
         }
         endpointRef = rpcEndpointRef.get();
       } while (endpointRef == null && index != currentIndex.get());
