@@ -24,20 +24,30 @@ import java.nio.ByteBuffer;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicLong;
 
+import javax.annotation.Nullable;
+
 import com.google.common.annotations.VisibleForTesting;
-import com.google.common.base.Objects;
 import com.google.common.base.Preconditions;
 import com.google.common.util.concurrent.SettableFuture;
 import io.netty.channel.Channel;
 import io.netty.channel.ChannelFuture;
 import io.netty.util.concurrent.Future;
 import io.netty.util.concurrent.GenericFutureListener;
+import org.apache.commons.lang3.builder.ToStringBuilder;
+import org.apache.commons.lang3.builder.ToStringStyle;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import org.apache.celeborn.common.network.buffer.NioManagedBuffer;
-import org.apache.celeborn.common.network.protocol.*;
+import org.apache.celeborn.common.network.protocol.OneWayMessage;
+import org.apache.celeborn.common.network.protocol.PushData;
+import org.apache.celeborn.common.network.protocol.PushMergedData;
+import org.apache.celeborn.common.network.protocol.RpcRequest;
+import org.apache.celeborn.common.network.protocol.StreamChunkSlice;
+import org.apache.celeborn.common.network.protocol.TransportMessage;
 import org.apache.celeborn.common.network.util.NettyUtils;
+import org.apache.celeborn.common.protocol.MessageType;
+import org.apache.celeborn.common.protocol.PbChunkFetchRequest;
 import org.apache.celeborn.common.read.FetchRequestInfo;
 import org.apache.celeborn.common.write.PushRequestInfo;
 
@@ -71,6 +81,7 @@ public class TransportClient implements Closeable {
   private final Channel channel;
   private final TransportResponseHandler handler;
   private volatile boolean timedOut;
+  @Nullable private String clientId;
 
   public TransportClient(Channel channel, TransportResponseHandler handler) {
     this.channel = Preconditions.checkNotNull(channel);
@@ -140,7 +151,19 @@ public class TransportClient implements Closeable {
     handler.addFetchRequest(streamChunkSlice, info);
 
     ChannelFuture channelFuture =
-        channel.writeAndFlush(new ChunkFetchRequest(streamChunkSlice)).addListener(listener);
+        channel
+            .writeAndFlush(
+                new RpcRequest(
+                    TransportClient.requestId(),
+                    new NioManagedBuffer(
+                        new TransportMessage(
+                                MessageType.CHUNK_FETCH_REQUEST,
+                                PbChunkFetchRequest.newBuilder()
+                                    .setStreamChunkSlice(streamChunkSlice.toProto())
+                                    .build()
+                                    .toByteArray())
+                            .toByteBuffer())))
+            .addListener(listener);
     info.setChannelFuture(channelFuture);
   }
 
@@ -169,19 +192,18 @@ public class TransportClient implements Closeable {
   }
 
   /**
-   * Sends an opaque message to the RpcHandler on the server-side.
+   * Sends an opaque message to the RpcHandler on the server-side. No reply is expected for the
+   * message, and no delivery guarantees are made.
    *
    * @param message The message to send.
-   * @return The RPC's id.
    */
-  public long sendRpc(ByteBuffer message) {
+  public void sendRpc(ByteBuffer message) {
     if (logger.isTraceEnabled()) {
       logger.trace("Sending RPC to {}", NettyUtils.getRemoteAddress(channel));
     }
 
     long requestId = requestId();
     channel.writeAndFlush(new RpcRequest(requestId, new NioManagedBuffer(message)));
-    return requestId;
   }
 
   public ChannelFuture pushData(
@@ -296,9 +318,10 @@ public class TransportClient implements Closeable {
 
   @Override
   public String toString() {
-    return Objects.toStringHelper(this)
-        .add("remoteAddress", channel.remoteAddress())
-        .add("isActive", isActive())
+    return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
+        .append("remoteAdress", channel.remoteAddress())
+        .append("clientId", clientId)
+        .append("isActive", isActive())
         .toString();
   }
 
@@ -308,6 +331,25 @@ public class TransportClient implements Closeable {
     return counter.getAndIncrement();
   }
 
+  /**
+   * Returns the ID used by the client to authenticate itself when authentication is enabled.
+   *
+   * @return The client ID, or null if authentication is disabled.
+   */
+  public String getClientId() {
+    return clientId;
+  }
+
+  /**
+   * Sets the authenticated client ID. This is meant to be used by the authentication layer.
+   *
+   * <p>Trying to set a different client ID after it's been set will result in an exception.
+   */
+  public void setClientId(String id) {
+    Preconditions.checkState(clientId == null, "Client ID has already been set.");
+    this.clientId = id;
+  }
+
   public class StdChannelListener implements GenericFutureListener<Future<? super Void>> {
     final long startTime;
     final Object requestId;
