@@ -17,14 +17,7 @@
 
 package org.apache.celeborn.common.network.protocol;
 
-import static org.apache.celeborn.common.protocol.MessageType.BACKLOG_ANNOUNCEMENT_VALUE;
-import static org.apache.celeborn.common.protocol.MessageType.BUFFER_STREAM_END_VALUE;
-import static org.apache.celeborn.common.protocol.MessageType.OPEN_STREAM_VALUE;
-import static org.apache.celeborn.common.protocol.MessageType.PUSH_DATA_HAND_SHAKE_VALUE;
-import static org.apache.celeborn.common.protocol.MessageType.READ_ADD_CREDIT_VALUE;
-import static org.apache.celeborn.common.protocol.MessageType.REGION_FINISH_VALUE;
-import static org.apache.celeborn.common.protocol.MessageType.REGION_START_VALUE;
-import static org.apache.celeborn.common.protocol.MessageType.STREAM_HANDLER_VALUE;
+import static org.apache.celeborn.common.protocol.MessageType.*;
 
 import java.io.Serializable;
 import java.nio.ByteBuffer;
@@ -38,17 +31,25 @@ import org.apache.celeborn.common.exception.CelebornIOException;
 import org.apache.celeborn.common.protocol.MessageType;
 import org.apache.celeborn.common.protocol.PbBacklogAnnouncement;
 import org.apache.celeborn.common.protocol.PbBufferStreamEnd;
+import org.apache.celeborn.common.protocol.PbChunkFetchRequest;
+import org.apache.celeborn.common.protocol.PbGetShuffleId;
+import org.apache.celeborn.common.protocol.PbGetShuffleIdResponse;
 import org.apache.celeborn.common.protocol.PbOpenStream;
 import org.apache.celeborn.common.protocol.PbPushDataHandShake;
 import org.apache.celeborn.common.protocol.PbReadAddCredit;
 import org.apache.celeborn.common.protocol.PbRegionFinish;
 import org.apache.celeborn.common.protocol.PbRegionStart;
+import org.apache.celeborn.common.protocol.PbReportShuffleFetchFailure;
+import org.apache.celeborn.common.protocol.PbReportShuffleFetchFailureResponse;
+import org.apache.celeborn.common.protocol.PbSaslRequest;
+import org.apache.celeborn.common.protocol.PbStreamChunkSlice;
 import org.apache.celeborn.common.protocol.PbStreamHandler;
+import org.apache.celeborn.common.protocol.PbTransportableError;
 
 public class TransportMessage implements Serializable {
   private static final long serialVersionUID = -3259000920699629773L;
   private static Logger logger = LoggerFactory.getLogger(TransportMessage.class);
-  @Deprecated private final MessageType type;
+  @Deprecated private final transient MessageType type;
   private final int messageTypeValue;
   private final byte[] payload;
 
@@ -76,18 +77,34 @@ public class TransportMessage implements Serializable {
         return (T) PbOpenStream.parseFrom(payload);
       case STREAM_HANDLER_VALUE:
         return (T) PbStreamHandler.parseFrom(payload);
-      case BACKLOG_ANNOUNCEMENT_VALUE:
-        return (T) PbBacklogAnnouncement.parseFrom(payload);
-      case BUFFER_STREAM_END_VALUE:
-        return (T) PbBufferStreamEnd.parseFrom(payload);
-      case READ_ADD_CREDIT_VALUE:
-        return (T) PbReadAddCredit.parseFrom(payload);
       case PUSH_DATA_HAND_SHAKE_VALUE:
         return (T) PbPushDataHandShake.parseFrom(payload);
       case REGION_START_VALUE:
         return (T) PbRegionStart.parseFrom(payload);
       case REGION_FINISH_VALUE:
         return (T) PbRegionFinish.parseFrom(payload);
+      case BACKLOG_ANNOUNCEMENT_VALUE:
+        return (T) PbBacklogAnnouncement.parseFrom(payload);
+      case BUFFER_STREAM_END_VALUE:
+        return (T) PbBufferStreamEnd.parseFrom(payload);
+      case READ_ADD_CREDIT_VALUE:
+        return (T) PbReadAddCredit.parseFrom(payload);
+      case STREAM_CHUNK_SLICE_VALUE:
+        return (T) PbStreamChunkSlice.parseFrom(payload);
+      case CHUNK_FETCH_REQUEST_VALUE:
+        return (T) PbChunkFetchRequest.parseFrom(payload);
+      case TRANSPORTABLE_ERROR_VALUE:
+        return (T) PbTransportableError.parseFrom(payload);
+      case GET_SHUFFLE_ID_VALUE:
+        return (T) PbGetShuffleId.parseFrom(payload);
+      case GET_SHUFFLE_ID_RESPONSE_VALUE:
+        return (T) PbGetShuffleIdResponse.parseFrom(payload);
+      case REPORT_SHUFFLE_FETCH_FAILURE_VALUE:
+        return (T) PbReportShuffleFetchFailure.parseFrom(payload);
+      case REPORT_SHUFFLE_FETCH_FAILURE_RESPONSE_VALUE:
+        return (T) PbReportShuffleFetchFailureResponse.parseFrom(payload);
+      case SASL_REQUEST_VALUE:
+        return (T) PbSaslRequest.parseFrom(payload);
       default:
         logger.error("Unexpected type {}", type);
     }
