@@ -45,16 +45,12 @@ import org.apache.celeborn.common.network.util.ByteUnit;
 /**
  * General utilities available in the network package. Many of these are sourced from Spark's own
  * Utils, just accessible within this package.
+ *
+ * <p>Some methods of this class are copied from Apache Spark
  */
 public class JavaUtils {
   private static final Logger logger = LoggerFactory.getLogger(JavaUtils.class);
 
-  /**
-   * Define a default value for driver memory here since this value is referenced across the code
-   * base and nearly all files already use Utils.scala
-   */
-  public static final long DEFAULT_DRIVER_MEM_MB = 1024;
-
   /** Closes the given object, ignoring IOExceptions. */
   public static void closeQuietly(Closeable closeable) {
     try {
