@@ -19,6 +19,7 @@ package org.apache.celeborn.common.meta
 
 import java.io.File
 import java.util
+import java.util.function.BiFunction
 
 import scala.collection.JavaConverters._
 import scala.collection.mutable.ListBuffer
@@ -49,6 +50,17 @@ class DiskInfo(
     this(mountPoint, usableSpace, avgFlushTime, avgFetchTime, activeSlots, List.empty, null)
   }
 
+  def this(
+      mountPoint: String,
+      usableSpace: Long,
+      avgFlushTime: Long,
+      avgFetchTime: Long,
+      activeSlots: Long,
+      storageType: StorageInfo.Type) = {
+    this(mountPoint, usableSpace, avgFlushTime, avgFetchTime, activeSlots, List.empty, null)
+    this.storageType = storageType
+  }
+
   def this(
       mountPoint: String,
       dirs: List[File],
@@ -70,9 +82,14 @@ class DiskInfo(
   var status: DiskStatus = DiskStatus.HEALTHY
   var threadCount = 1
   var configuredUsableSpace = 0L
-  var storageType: StorageInfo.Type = _
+  var storageType: StorageInfo.Type = StorageInfo.Type.SSD
   var maxSlots: Long = 0
   lazy val shuffleAllocations = new util.HashMap[String, Integer]()
+  lazy val applicationAllocations = new util.HashMap[String, Integer]()
+
+  def setStorageType(storageType: StorageInfo.Type) = {
+    this.storageType = storageType
+  }
 
   def setStatus(status: DiskStatus): this.type = this.synchronized {
     this.status = status
@@ -106,28 +123,38 @@ class DiskInfo(
   }
 
   def allocateSlots(shuffleKey: String, slots: Int): Unit = this.synchronized {
-    val allocated = shuffleAllocations.getOrDefault(shuffleKey, 0)
-    shuffleAllocations.put(shuffleKey, allocated + slots)
+    val applicationId = Utils.splitShuffleKey(shuffleKey)._1
+    val shuffleAllocated = shuffleAllocations.getOrDefault(shuffleKey, 0)
+    val applicationAllocated = applicationAllocations.getOrDefault(applicationId, 0)
+    shuffleAllocations.put(shuffleKey, shuffleAllocated + slots)
+    applicationAllocations.put(applicationId, applicationAllocated + slots)
     activeSlots = activeSlots + slots
   }
 
   def releaseSlots(shuffleKey: String, slots: Int): Unit = this.synchronized {
-    val allocated = shuffleAllocations.getOrDefault(shuffleKey, 0)
-    if (allocated < slots) {
-      logError(s"allocated $allocated is less than to release $slots !")
-    }
-    val delta = Math.min(allocated, slots)
-    activeSlots = activeSlots - delta
-    if (allocated > slots) {
-      shuffleAllocations.put(shuffleKey, allocated - slots)
+    val applicationId = Utils.splitShuffleKey(shuffleKey)._1
+    val shuffleAllocated = shuffleAllocations.getOrDefault(shuffleKey, 0)
+    val applicationAllocated = applicationAllocations.getOrDefault(applicationId, 0)
+    if (shuffleAllocated < slots) {
+      logError(s"allocated $shuffleAllocated is less than to release $slots !")
     } else {
-      shuffleAllocations.put(shuffleKey, 0)
+      shuffleAllocations.put(shuffleKey, shuffleAllocated - slots)
+      applicationAllocations.put(applicationId, applicationAllocated - slots)
     }
+    activeSlots = activeSlots - Math.min(shuffleAllocated, slots)
   }
 
   def releaseSlots(shuffleKey: String): Unit = this.synchronized {
     val allocated = shuffleAllocations.remove(shuffleKey)
     if (allocated != null) {
+      val applicationId = Utils.splitShuffleKey(shuffleKey)._1
+      var applicationAllocated = applicationAllocations.getOrDefault(applicationId, 0)
+      applicationAllocated = applicationAllocated - allocated
+      if (applicationAllocated <= 0) {
+        applicationAllocations.remove(applicationId)
+      } else {
+        applicationAllocations.put(applicationId, applicationAllocated)
+      }
       activeSlots = activeSlots - allocated
     }
   }
@@ -136,18 +163,25 @@ class DiskInfo(
     new util.HashSet(shuffleAllocations.keySet())
   }
 
+  def getApplicationIdSet(): util.HashSet[String] = this.synchronized {
+    new util.HashSet(applicationAllocations.keySet())
+  }
+
   override def toString: String = this.synchronized {
     val (emptyShuffles, nonEmptyShuffles) = shuffleAllocations.asScala.partition(_._2 == 0)
     s"DiskInfo(maxSlots: $maxSlots," +
-      s" committed shuffles ${emptyShuffles.size}" +
+      s" committed shuffles ${emptyShuffles.size}," +
+      s" running applications ${applicationAllocations.size}," +
       s" shuffleAllocations: ${nonEmptyShuffles.toMap}," +
       s" mountPoint: $mountPoint," +
       s" usableSpace: ${Utils.bytesToString(actualUsableSpace)}," +
       s" avgFlushTime: ${Utils.nanoDurationToString(avgFlushTime)}," +
       s" avgFetchTime: ${Utils.nanoDurationToString(avgFetchTime)}," +
-      s" activeSlots: $activeSlots)" +
+      s" activeSlots: $activeSlots," +
+      s" storageType: ${storageType})" +
       s" status: $status" +
       s" dirs ${dirs.mkString("\t")}"
+
   }
 }
 
