@@ -63,14 +63,17 @@ object PbSerDeUtils {
       .setMinor(minor)
       .build.toByteArray
 
-  def fromPbDiskInfo(pbDiskInfo: PbDiskInfo): DiskInfo =
-    new DiskInfo(
+  def fromPbDiskInfo(pbDiskInfo: PbDiskInfo): DiskInfo = {
+    val diskInfo = new DiskInfo(
       pbDiskInfo.getMountPoint,
       pbDiskInfo.getUsableSpace,
       pbDiskInfo.getAvgFlushTime,
       pbDiskInfo.getAvgFetchTime,
       pbDiskInfo.getUsedSlots)
       .setStatus(Utils.toDiskStatus(pbDiskInfo.getStatus))
+    diskInfo.setStorageType(StorageInfo.typesMap.get(pbDiskInfo.getStorageType))
+    diskInfo
+  }
 
   def toPbDiskInfo(diskInfo: DiskInfo): PbDiskInfo =
     PbDiskInfo.newBuilder
@@ -80,6 +83,7 @@ object PbSerDeUtils {
       .setAvgFetchTime(diskInfo.avgFetchTime)
       .setUsedSlots(diskInfo.activeSlots)
       .setStatus(diskInfo.status.getValue)
+      .setStorageType(diskInfo.storageType.getValue)
       .build
 
   def fromPbFileInfo(pbFileInfo: PbFileInfo): FileInfo =
@@ -369,6 +373,7 @@ object PbSerDeUtils {
       registeredShuffle: java.util.Set[String],
       hostnameSet: java.util.Set[String],
       excludedWorkers: java.util.Set[WorkerInfo],
+      manuallyExcludedWorkers: java.util.Set[WorkerInfo],
       workerLostEvent: java.util.Set[WorkerInfo],
       appHeartbeatTime: java.util.Map[String, java.lang.Long],
       workers: java.util.List[WorkerInfo],
@@ -383,6 +388,8 @@ object PbSerDeUtils {
       .addAllRegisteredShuffle(registeredShuffle)
       .addAllHostnameSet(hostnameSet)
       .addAllExcludedWorkers(excludedWorkers.asScala.map(toPbWorkerInfo(_, true)).asJava)
+      .addAllManuallyExcludedWorkers(manuallyExcludedWorkers.asScala
+        .map(toPbWorkerInfo(_, true)).asJava)
       .addAllWorkerLostEvents(workerLostEvent.asScala.map(toPbWorkerInfo(_, true)).asJava)
       .putAllAppHeartbeatTime(appHeartbeatTime)
       .addAllWorkers(workers.asScala.map(toPbWorkerInfo(_, true)).asJava)
