@@ -63,6 +63,7 @@ public abstract class AbstractMetaManager implements IMetadataHandler {
   public final ConcurrentHashMap<WorkerInfo, Long> lostWorkers = JavaUtils.newConcurrentHashMap();
   public final ConcurrentHashMap<String, Long> appHeartbeatTime = JavaUtils.newConcurrentHashMap();
   public final Set<WorkerInfo> excludedWorkers = ConcurrentHashMap.newKeySet();
+  public final Set<WorkerInfo> manuallyExcludedWorkers = ConcurrentHashMap.newKeySet();
   public final Set<WorkerInfo> shutdownWorkers = ConcurrentHashMap.newKeySet();
   public final Set<WorkerInfo> workerLostEvents = ConcurrentHashMap.newKeySet();
 
@@ -111,6 +112,12 @@ public abstract class AbstractMetaManager implements IMetadataHandler {
     appHeartbeatTime.remove(appId);
   }
 
+  public void updateWorkerExcludeMeta(
+      List<WorkerInfo> workersToAdd, List<WorkerInfo> workersToRemove) {
+    manuallyExcludedWorkers.addAll(workersToAdd);
+    workersToRemove.forEach(manuallyExcludedWorkers::remove);
+  }
+
   public void updateWorkerLostMeta(
       String host, int rpcPort, int pushPort, int fetchPort, int replicatePort) {
     WorkerInfo worker = new WorkerInfo(host, rpcPort, pushPort, fetchPort, replicatePort);
@@ -223,6 +230,7 @@ public abstract class AbstractMetaManager implements IMetadataHandler {
                 registeredShuffle,
                 hostnameSet,
                 excludedWorkers,
+                manuallyExcludedWorkers,
                 workerLostEvents,
                 appHeartbeatTime,
                 workers,
@@ -255,6 +263,10 @@ public abstract class AbstractMetaManager implements IMetadataHandler {
           snapshotMetaInfo.getExcludedWorkersList().stream()
               .map(PbSerDeUtils::fromPbWorkerInfo)
               .collect(Collectors.toSet()));
+      manuallyExcludedWorkers.addAll(
+          snapshotMetaInfo.getManuallyExcludedWorkersList().stream()
+              .map(PbSerDeUtils::fromPbWorkerInfo)
+              .collect(Collectors.toSet()));
       workerLostEvents.addAll(
           snapshotMetaInfo.getWorkerLostEventsList().stream()
               .map(PbSerDeUtils::fromPbWorkerInfo)
@@ -309,10 +321,11 @@ public abstract class AbstractMetaManager implements IMetadataHandler {
     }
     LOG.info("Successfully restore meta info from snapshot {}", file.getAbsolutePath());
     LOG.info(
-        "Worker size: {}, Registered shuffle size: {}. Worker excluded list size: {}.",
+        "Worker size: {}, Registered shuffle size: {}. Worker excluded list size: {}. Manually Excluded list size: {}",
         workers.size(),
         registeredShuffle.size(),
-        excludedWorkers.size());
+        excludedWorkers.size(),
+        manuallyExcludedWorkers.size());
     workers.forEach(workerInfo -> LOG.info(workerInfo.toString()));
     registeredShuffle.forEach(shuffle -> LOG.info("RegisteredShuffle {}", shuffle));
   }
@@ -325,6 +338,7 @@ public abstract class AbstractMetaManager implements IMetadataHandler {
     appHeartbeatTime.clear();
     excludedWorkers.clear();
     shutdownWorkers.clear();
+    manuallyExcludedWorkers.clear();
     workerLostEvents.clear();
     partitionTotalWritten.reset();
     partitionTotalFileCount.reset();
@@ -354,7 +368,9 @@ public abstract class AbstractMetaManager implements IMetadataHandler {
         Utils.bytesToString(oldEstimatedPartitionSize),
         Utils.bytesToString(estimatedPartitionSize));
     workers.stream()
-        .filter(worker -> !excludedWorkers.contains(worker))
+        .filter(
+            worker ->
+                !excludedWorkers.contains(worker) && !manuallyExcludedWorkers.contains(worker))
         .forEach(workerInfo -> workerInfo.updateDiskMaxSlots(estimatedPartitionSize));
   }
 }
