@@ -85,7 +85,7 @@ public class HARaftServer {
   private final StateMachine masterStateMachine;
 
   private final ScheduledExecutorService scheduledRoleChecker =
-      ThreadUtils.newDaemonSingleThreadScheduledExecutor("ratis-role-checker");
+      ThreadUtils.newDaemonSingleThreadScheduledExecutor("master-ratis-role-checker");
   private long roleCheckIntervalMs;
   private final ReentrantReadWriteLock roleCheckLock = new ReentrantReadWriteLock();
   private Optional<RaftProtos.RaftPeerRole> cachedPeerRole = Optional.empty();
@@ -496,7 +496,7 @@ public class HARaftServer {
     }
   }
 
-  private GroupInfoReply getGroupInfo() throws IOException {
+  public GroupInfoReply getGroupInfo() throws IOException {
     GroupInfoRequest groupInfoRequest =
         new GroupInfoRequest(clientId, raftPeerId, RAFT_GROUP_ID, nextCallId());
     return server.getGroupInfo(groupInfoRequest);
