@@ -27,6 +27,8 @@ import scala.collection.JavaConverters._
 import scala.util.Random
 
 import org.apache.hadoop.fs.{FileSystem, Path}
+import org.apache.ratis.proto.RaftProtos
+import org.apache.ratis.proto.RaftProtos.RaftPeerRole
 
 import org.apache.celeborn.common.CelebornConf
 import org.apache.celeborn.common.client.MasterClient
@@ -34,12 +36,13 @@ import org.apache.celeborn.common.identity.UserIdentifier
 import org.apache.celeborn.common.internal.Logging
 import org.apache.celeborn.common.meta.{DiskInfo, WorkerInfo}
 import org.apache.celeborn.common.metrics.MetricsSystem
-import org.apache.celeborn.common.metrics.source.{JVMCPUSource, JVMSource, ResourceConsumptionSource, SystemMiscSource}
+import org.apache.celeborn.common.metrics.source.{JVMCPUSource, JVMSource, ResourceConsumptionSource, SystemMiscSource, ThreadPoolSource}
 import org.apache.celeborn.common.protocol._
 import org.apache.celeborn.common.protocol.message.{ControlMessages, StatusCode}
 import org.apache.celeborn.common.protocol.message.ControlMessages._
 import org.apache.celeborn.common.quota.{QuotaManager, ResourceConsumption}
 import org.apache.celeborn.common.rpc._
+import org.apache.celeborn.common.rpc.netty.NettyRpcEnv
 import org.apache.celeborn.common.util.{CelebornHadoopUtils, JavaUtils, PbSerDeUtils, ThreadUtils, Utils}
 import org.apache.celeborn.server.common.{HttpService, Service}
 import org.apache.celeborn.service.deploy.master.clustermeta.SingleMasterMetaManager
@@ -55,7 +58,18 @@ private[celeborn] class Master(
   override def serviceName: String = Service.MASTER
 
   override val metricsSystem: MetricsSystem =
-    MetricsSystem.createMetricsSystem(serviceName, conf, MetricsSystem.SERVLET_PATH)
+    MetricsSystem.createMetricsSystem(serviceName, conf)
+  // init and register master metrics
+  private val resourceConsumptionSource =
+    new ResourceConsumptionSource(conf, MetricsSystem.ROLE_MASTER)
+  private val threadPoolSource = ThreadPoolSource(conf, MetricsSystem.ROLE_MASTER)
+  private val masterSource = new MasterSource(conf)
+  metricsSystem.registerSource(resourceConsumptionSource)
+  metricsSystem.registerSource(masterSource)
+  metricsSystem.registerSource(threadPoolSource)
+  metricsSystem.registerSource(new JVMSource(conf, MetricsSystem.ROLE_MASTER))
+  metricsSystem.registerSource(new JVMCPUSource(conf, MetricsSystem.ROLE_MASTER))
+  metricsSystem.registerSource(new SystemMiscSource(conf, MetricsSystem.ROLE_MASTER))
 
   override val rpcEnv: RpcEnv = RpcEnv.create(
     RpcNameConstants.MASTER_SYS,
@@ -90,7 +104,7 @@ private[celeborn] class Master(
 
   // Threads
   private val forwardMessageThread =
-    ThreadUtils.newDaemonSingleThreadScheduledExecutor("master-forward-message-thread")
+    ThreadUtils.newDaemonSingleThreadScheduledExecutor("master-message-forwarder")
   private var checkForWorkerTimeOutTask: ScheduledFuture[_] = _
   private var checkForApplicationTimeOutTask: ScheduledFuture[_] = _
   private var checkForUnavailableWorkerTimeOutTask: ScheduledFuture[_] = _
@@ -133,7 +147,7 @@ private[celeborn] class Master(
   private val estimatedPartitionSizeForEstimationUpdateInterval =
     conf.estimatedPartitionSizeForEstimationUpdateInterval
   private val partitionSizeUpdateService =
-    ThreadUtils.newDaemonSingleThreadScheduledExecutor("partition-size-updater")
+    ThreadUtils.newDaemonSingleThreadScheduledExecutor("master-partition-size-updater")
   partitionSizeUpdateService.scheduleWithFixedDelay(
     new Runnable {
       override def run(): Unit = {
@@ -149,15 +163,12 @@ private[celeborn] class Master(
     TimeUnit.MILLISECONDS)
   private val slotsAssignPolicy = conf.masterSlotAssignPolicy
 
-  // init and register master metrics
-  val resourceConsumptionSource = new ResourceConsumptionSource(conf)
-  private val masterSource = new MasterSource(conf)
   private var hadoopFs: FileSystem = _
   masterSource.addGauge(MasterSource.REGISTERED_SHUFFLE_COUNT) { () =>
     statusSystem.registeredShuffle.size
   }
   masterSource.addGauge(MasterSource.EXCLUDED_WORKER_COUNT) { () =>
-    statusSystem.excludedWorkers.size
+    statusSystem.excludedWorkers.size + statusSystem.manuallyExcludedWorkers.size
   }
   masterSource.addGauge(MasterSource.WORKER_COUNT) { () => statusSystem.workers.size }
   masterSource.addGauge(MasterSource.LOST_WORKER_COUNT) { () => statusSystem.lostWorkers.size }
@@ -165,7 +176,7 @@ private[celeborn] class Master(
     statusSystem.appHeartbeatTime.size
   }
   masterSource.addGauge(MasterSource.PARTITION_SIZE) { () => statusSystem.estimatedPartitionSize }
-  masterSource.addGauge(MasterSource.PARTITION_WRITTEN) { () =>
+  masterSource.addGauge(MasterSource.ACTIVE_SHUFFLE_SIZE) { () =>
     statusSystem.workers.parallelStream()
       .mapToLong(new ToLongFunction[WorkerInfo]() {
         override def applyAsLong(value: WorkerInfo): Long =
@@ -175,7 +186,7 @@ private[celeborn] class Master(
             }).sum()
       }).sum()
   }
-  masterSource.addGauge(MasterSource.PARTITION_FILE_COUNT) { () =>
+  masterSource.addGauge(MasterSource.ACTIVE_SHUFFLE_FILE_COUNT) { () =>
     statusSystem.workers.parallelStream()
       .mapToLong(new ToLongFunction[WorkerInfo]() {
         override def applyAsLong(value: WorkerInfo): Long =
@@ -187,12 +198,6 @@ private[celeborn] class Master(
   }
   masterSource.addGauge(MasterSource.IS_ACTIVE_MASTER) { () => isMasterActive }
 
-  metricsSystem.registerSource(resourceConsumptionSource)
-  metricsSystem.registerSource(masterSource)
-  metricsSystem.registerSource(new JVMSource(conf, MetricsSystem.ROLE_MASTER))
-  metricsSystem.registerSource(new JVMCPUSource(conf, MetricsSystem.ROLE_MASTER))
-  metricsSystem.registerSource(new SystemMiscSource(conf, MetricsSystem.ROLE_MASTER))
-
   rpcEnv.setupEndpoint(RpcNameConstants.MASTER_EP, this)
 
   // start threads to check timeout for workers and applications
@@ -354,7 +359,7 @@ private[celeborn] class Master(
       // keep it for compatible reason
       context.reply(ReleaseSlotsResponse(StatusCode.SUCCESS))
 
-    case requestSlots @ RequestSlots(_, _, _, _, _, _, _, _, _, _) =>
+    case requestSlots @ RequestSlots(_, _, _, _, _, _, _, _, _, _, _) =>
       logTrace(s"Received RequestSlots request $requestSlots.")
       executeWithLeaderChecker(context, handleRequestSlots(context, requestSlots))
 
@@ -367,9 +372,6 @@ private[celeborn] class Master(
         context,
         handleUnregisterShuffle(context, applicationId, shuffleId, requestId))
 
-    case msg: GetBlacklist =>
-      executeWithLeaderChecker(context, handleGetBlacklist(context, msg))
-
     case ApplicationLost(appId, requestId) =>
       logDebug(
         s"Received ApplicationLost request $requestId, $appId from ${context.senderAddress}.")
@@ -410,6 +412,19 @@ private[celeborn] class Master(
         context,
         handleReportNodeUnavailable(context, failedWorkers, requestId))
 
+    case pb: PbWorkerExclude =>
+      val workersToAdd = new util.ArrayList[WorkerInfo](pb.getWorkersToAddList
+        .asScala.map(PbSerDeUtils.fromPbWorkerInfo).toList.asJava)
+      val workersToRemove = new util.ArrayList[WorkerInfo](pb.getWorkersToRemoveList
+        .asScala.map(PbSerDeUtils.fromPbWorkerInfo).toList.asJava)
+      executeWithLeaderChecker(
+        context,
+        handleWorkerExclude(
+          context,
+          workersToAdd,
+          workersToRemove,
+          pb.getRequestId))
+
     case pb: PbWorkerLost =>
       val host = pb.getHost
       val rpcPort = pb.getRpcPort
@@ -429,14 +444,13 @@ private[celeborn] class Master(
       executeWithLeaderChecker(context, handleCheckWorkersAvailable(context))
   }
 
-  private def timeoutDeadWorkers() {
+  private def timeoutDeadWorkers(): Unit = {
     val currentTime = System.currentTimeMillis()
     // Need increase timeout deadline to avoid long time leader election period
     if (HAHelper.getWorkerTimeoutDeadline(statusSystem) > currentTime) {
       return
     }
 
-    var ind = 0
     workersSnapShot.asScala.foreach { worker =>
       if (worker.lastHeartbeat < currentTime - workerHeartbeatTimeoutMs
         && !statusSystem.workerLostEvents.contains(worker)) {
@@ -450,7 +464,6 @@ private[celeborn] class Master(
           worker.replicatePort,
           MasterClient.genRequestId()))
       }
-      ind += 1
     }
   }
 
@@ -469,7 +482,7 @@ private[celeborn] class Master(
       logDebug(s"Remove unavailable info for workers: $unavailableInfoTimeoutWorkers")
       self.send(RemoveWorkersUnavailableInfo(
         unavailableInfoTimeoutWorkers,
-        MasterClient.genRequestId()));
+        MasterClient.genRequestId()))
     }
   }
 
@@ -540,6 +553,20 @@ private[celeborn] class Master(
     context.reply(HeartbeatFromWorkerResponse(expiredShuffleKeys, registered))
   }
 
+  private def handleWorkerExclude(
+      context: RpcCallContext,
+      workersToAdd: util.List[WorkerInfo],
+      workersToRemove: util.List[WorkerInfo],
+      requestId: String): Unit = {
+    statusSystem.handleWorkerExclude(
+      workersToAdd.asScala.filter(workersSnapShot.contains(_)).asJava,
+      workersToRemove.asScala.filter(workersSnapShot.contains(_)).asJava,
+      requestId)
+    if (context != null) {
+      context.reply(WorkerExcludeResponse(true))
+    }
+  }
+
   private def handleWorkerLost(
       context: RpcCallContext,
       host: String,
@@ -639,8 +666,14 @@ private[celeborn] class Master(
     val numReducers = requestSlots.partitionIdList.size()
     val shuffleKey = Utils.makeShuffleKey(requestSlots.applicationId, requestSlots.shuffleId)
 
-    val availableWorkers = workersAvailable()
+    val availableWorkers = workersAvailable(requestSlots.excludedWorkerSet)
     val numAvailableWorkers = availableWorkers.size()
+
+    if (numAvailableWorkers == 0) {
+      logError(s"Offer slots for $shuffleKey failed due to all workers are excluded!")
+      context.reply(RequestSlotsResponse(StatusCode.WORKER_EXCLUDED, new WorkerResource()))
+    }
+
     val numWorkers = Math.min(
       Math.max(
         if (requestSlots.shouldReplicate) 2 else 1,
@@ -661,7 +694,7 @@ private[celeborn] class Master(
     val slots =
       masterSource.sample(MasterSource.OFFER_SLOTS_TIME, s"offerSlots-${Random.nextInt()}") {
         statusSystem.workers.synchronized {
-          if (slotsAssignPolicy == SlotsAssignPolicy.LOADAWARE && !hasHDFSStorage) {
+          if (slotsAssignPolicy == SlotsAssignPolicy.LOADAWARE) {
             SlotsAllocator.offerSlotsLoadAware(
               selectedWorkers,
               requestSlots.partitionIdList,
@@ -736,15 +769,6 @@ private[celeborn] class Master(
     context.reply(UnregisterShuffleResponse(StatusCode.SUCCESS))
   }
 
-  def handleGetBlacklist(context: RpcCallContext, msg: GetBlacklist): Unit = {
-    msg.localExcludedWorkers.removeAll(workersSnapShot)
-    context.reply(
-      GetBlacklistResponse(
-        StatusCode.SUCCESS,
-        new util.ArrayList(statusSystem.excludedWorkers),
-        msg.localExcludedWorkers))
-  }
-
   private def handleReportNodeUnavailable(
       context: RpcCallContext,
       failedWorkers: util.List[WorkerInfo],
@@ -780,7 +804,7 @@ private[celeborn] class Master(
     }
     val hdfsWorkPath = new Path(conf.hdfsDir, conf.workerWorkingDir)
     if (hadoopFs.exists(hdfsWorkPath)) {
-      if (!expiredDir.isEmpty) {
+      if (expiredDir.nonEmpty) {
         val dirToDelete = new Path(hdfsWorkPath, expiredDir)
         // delete specific app dir on application lost
         CelebornHadoopUtils.deleteHDFSPathOrLogError(hadoopFs, dirToDelete, true)
@@ -826,7 +850,7 @@ private[celeborn] class Master(
   private def handleRemoveWorkersUnavailableInfos(
       unavailableWorkers: util.List[WorkerInfo],
       requestId: String): Unit = {
-    statusSystem.handleRemoveWorkersUnavailableInfo(unavailableWorkers, requestId);
+    statusSystem.handleRemoveWorkersUnavailableInfo(unavailableWorkers, requestId)
   }
 
   private def computeUserResourceConsumption(userIdentifier: UserIdentifier)
@@ -856,16 +880,24 @@ private[celeborn] class Master(
       userIdentifier: UserIdentifier,
       context: RpcCallContext): Unit = {
 
-    resourceConsumptionSource.addGauge("diskFileCount", userIdentifier.toMap) { () =>
+    resourceConsumptionSource.addGauge(
+      ResourceConsumptionSource.DISK_FILE_COUNT,
+      userIdentifier.toMap) { () =>
       computeUserResourceConsumption(userIdentifier).diskFileCount
     }
-    resourceConsumptionSource.addGauge("diskBytesWritten", userIdentifier.toMap) { () =>
+    resourceConsumptionSource.addGauge(
+      ResourceConsumptionSource.DISK_BYTES_WRITTEN,
+      userIdentifier.toMap) { () =>
       computeUserResourceConsumption(userIdentifier).diskBytesWritten
     }
-    resourceConsumptionSource.addGauge("hdfsFileCount", userIdentifier.toMap) { () =>
+    resourceConsumptionSource.addGauge(
+      ResourceConsumptionSource.HDFS_FILE_COUNT,
+      userIdentifier.toMap) { () =>
       computeUserResourceConsumption(userIdentifier).hdfsFileCount
     }
-    resourceConsumptionSource.addGauge("hdfsBytesWritten", userIdentifier.toMap) { () =>
+    resourceConsumptionSource.addGauge(
+      ResourceConsumptionSource.HDFS_BYTES_WRITTEN,
+      userIdentifier.toMap) { () =>
       computeUserResourceConsumption(userIdentifier).hdfsBytesWritten
     }
 
@@ -883,17 +915,26 @@ private[celeborn] class Master(
   private def workersAvailable(
       tmpExcludedWorkerList: Set[WorkerInfo] = Set.empty): util.List[WorkerInfo] = {
     workersSnapShot.asScala.filter { w =>
-      !statusSystem.excludedWorkers.contains(w) && !statusSystem.shutdownWorkers.contains(
-        w) && !tmpExcludedWorkerList.contains(w)
+      !statusSystem.excludedWorkers.contains(w) && !statusSystem.manuallyExcludedWorkers.contains(
+        w) && !statusSystem.shutdownWorkers.contains(w) && !tmpExcludedWorkerList.contains(w)
     }.asJava
   }
 
+  override def getMasterGroupInfo: String = {
+    val sb = new StringBuilder
+    sb.append("====================== Master Group INFO ==============================\n")
+    sb.append(getMasterGroupInfoInternal)
+    sb.toString()
+  }
+
+  private def getWorkers: String = {
+    workersSnapShot.asScala.mkString("\n")
+  }
+
   override def getWorkerInfo: String = {
     val sb = new StringBuilder
     sb.append("====================== Workers Info in Master =========================\n")
-    workersSnapShot.asScala.foreach { w =>
-      sb.append(w).append("\n")
-    }
+    sb.append(getWorkers)
     sb.toString()
   }
 
@@ -901,7 +942,7 @@ private[celeborn] class Master(
     val sb = new StringBuilder
     sb.append("======================= Lost Workers in Master ========================\n")
     lostWorkersSnapshot.asScala.toSeq.sortBy(_._2).foreach { case (worker, time) =>
-      sb.append(s"${worker.toUniqueId().padTo(50, " ").mkString}${dateFmt.format(time)}\n")
+      sb.append(s"${worker.toUniqueId().padTo(50, " ").mkString}${Utils.formatTimestamp(time)}\n")
     }
     sb.toString()
   }
@@ -918,8 +959,8 @@ private[celeborn] class Master(
   override def getExcludedWorkers: String = {
     val sb = new StringBuilder
     sb.append("===================== Excluded Workers in Master ======================\n")
-    statusSystem.excludedWorkers.asScala.foreach { worker =>
-      sb.append(s"${worker.toUniqueId()}\n")
+    (statusSystem.excludedWorkers.asScala ++ statusSystem.manuallyExcludedWorkers.asScala).foreach {
+      worker => sb.append(s"${worker.toUniqueId()}\n")
     }
     sb.toString()
   }
@@ -944,7 +985,7 @@ private[celeborn] class Master(
     val sb = new StringBuilder
     sb.append("================= LifecycleManager Application List ======================\n")
     statusSystem.appHeartbeatTime.asScala.toSeq.sortBy(_._2).foreach { case (appId, time) =>
-      sb.append(s"${appId.padTo(40, " ").mkString}${dateFmt.format(time)}\n")
+      sb.append(s"${appId.padTo(40, " ").mkString}${Utils.formatTimestamp(time)}\n")
     }
     sb.toString()
   }
@@ -965,13 +1006,30 @@ private[celeborn] class Master(
     sb.toString()
   }
 
-  override def listPartitionLocationInfo: String = throw new UnsupportedOperationException()
-
-  override def getUnavailablePeers: String = throw new UnsupportedOperationException()
-
-  override def isShutdown: String = throw new UnsupportedOperationException()
-
-  override def isRegistered: String = throw new UnsupportedOperationException()
+  override def exclude(addWorkers: String, removeWorkers: String): String = {
+    val sb = new StringBuilder
+    sb.append("============================ Add/Remove Excluded Workers  Manually =============================\n")
+    val workersToAdd = addWorkers.split(",").filter(_.nonEmpty)
+    val workersToRemove = removeWorkers.split(",").filter(_.nonEmpty)
+    val workerExcludeResponse = self.askSync[PbWorkerExcludeResponse](WorkerExclude(
+      workersToAdd.map(WorkerInfo.fromUniqueId).toList.asJava,
+      workersToRemove.map(WorkerInfo.fromUniqueId).toList.asJava,
+      MasterClient.genRequestId()))
+    if (workerExcludeResponse.getSuccess) {
+      sb.append(
+        s"Excluded workers add ${workersToAdd.mkString(",")} and remove ${workersToRemove.mkString(",")} successfully.\n")
+    } else {
+      sb.append(
+        s"Failed to Exclude workers add ${workersToAdd.mkString(",")} and remove ${workersToRemove.mkString(",")}.\n")
+    }
+    val unknownExcludedWorkers =
+      (workersToAdd ++ workersToRemove).filter(!workersSnapShot.contains(_))
+    if (unknownExcludedWorkers.nonEmpty) {
+      sb.append(
+        s"Unknown worker ${unknownExcludedWorkers.mkString(",")}. Workers in Master:\n$getWorkers.")
+    }
+    sb.toString()
+  }
 
   private def isMasterActive: Int = {
     // use int rather than bool for better monitoring on dashboard
@@ -988,6 +1046,40 @@ private[celeborn] class Master(
     isActive
   }
 
+  private def getMasterGroupInfoInternal: String = {
+    if (conf.haEnabled) {
+      val sb = new StringBuilder
+      val groupInfo = statusSystem.asInstanceOf[HAMasterMetaManager].getRatisServer.getGroupInfo
+      sb.append(s"group id: ${groupInfo.getGroup.getGroupId.getUuid}\n")
+
+      def getLeader(roleInfo: RaftProtos.RoleInfoProto): RaftProtos.RaftPeerProto = {
+        if (roleInfo == null) {
+          return null
+        }
+        if (roleInfo.getRole == RaftPeerRole.LEADER) {
+          return roleInfo.getSelf
+        }
+        val followerInfo = roleInfo.getFollowerInfo
+        if (followerInfo == null) {
+          return null
+        }
+        followerInfo.getLeaderInfo.getId
+      }
+
+      val leader = getLeader(groupInfo.getRoleInfoProto)
+      if (leader == null) {
+        sb.append("leader not found\n")
+      } else {
+        sb.append(s"leader info: ${leader.getId.toStringUtf8}(${leader.getAddress})\n\n")
+      }
+      sb.append(groupInfo.getCommitInfos)
+      sb.append("\n")
+      sb.toString()
+    } else {
+      "HA is not enabled"
+    }
+  }
+
   override def initialize(): Unit = {
     super.initialize()
     logInfo("Master started.")
