@@ -17,6 +17,8 @@
 
 package org.apache.celeborn.service.deploy.master.clustermeta;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertTrue;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.when;
@@ -25,7 +27,6 @@ import java.util.*;
 import java.util.concurrent.atomic.AtomicLong;
 
 import org.junit.After;
-import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
 
@@ -42,43 +43,43 @@ import org.apache.celeborn.common.rpc.netty.NettyRpcEndpointRef;
 
 public class DefaultMetaSystemSuiteJ {
 
-  private RpcEnv mockRpcEnv = mock(RpcEnv.class);
-  private CelebornConf conf = new CelebornConf();
+  private final RpcEnv mockRpcEnv = mock(RpcEnv.class);
+  private final CelebornConf conf = new CelebornConf();
   private AbstractMetaManager statusSystem;
-  private RpcEndpointRef dummyRef =
+  private final RpcEndpointRef dummyRef =
       new NettyRpcEndpointRef(
           new CelebornConf(), RpcEndpointAddress.apply("localhost", 111, "dummy"), null);
-  private AtomicLong callerId = new AtomicLong();
+  private final AtomicLong callerId = new AtomicLong();
 
-  private static String HOSTNAME1 = "host1";
-  private static int RPCPORT1 = 1111;
-  private static int PUSHPORT1 = 1112;
-  private static int FETCHPORT1 = 1113;
-  private static int REPLICATEPORT1 = 1114;
-  private static Map<String, DiskInfo> disks1 = new HashMap<>();
-  private static Map<UserIdentifier, ResourceConsumption> userResourceConsumption1 =
+  private static final String HOSTNAME1 = "host1";
+  private static final int RPCPORT1 = 1111;
+  private static final int PUSHPORT1 = 1112;
+  private static final int FETCHPORT1 = 1113;
+  private static final int REPLICATEPORT1 = 1114;
+  private static final Map<String, DiskInfo> disks1 = new HashMap<>();
+  private static final Map<UserIdentifier, ResourceConsumption> userResourceConsumption1 =
       new HashMap<>();
 
-  private static String HOSTNAME2 = "host2";
-  private static int RPCPORT2 = 2111;
-  private static int PUSHPORT2 = 2112;
-  private static int FETCHPORT2 = 2113;
-  private static int REPLICATEPORT2 = 2114;
-  private static Map<String, DiskInfo> disks2 = new HashMap<>();
-  private static Map<UserIdentifier, ResourceConsumption> userResourceConsumption2 =
+  private static final String HOSTNAME2 = "host2";
+  private static final int RPCPORT2 = 2111;
+  private static final int PUSHPORT2 = 2112;
+  private static final int FETCHPORT2 = 2113;
+  private static final int REPLICATEPORT2 = 2114;
+  private static final Map<String, DiskInfo> disks2 = new HashMap<>();
+  private static final Map<UserIdentifier, ResourceConsumption> userResourceConsumption2 =
       new HashMap<>();
 
-  private static String HOSTNAME3 = "host3";
-  private static int RPCPORT3 = 3111;
-  private static int PUSHPORT3 = 3112;
-  private static int FETCHPORT3 = 3113;
-  private static int REPLICATEPORT3 = 3114;
-  private static Map<String, DiskInfo> disks3 = new HashMap<>();
-  private static Map<UserIdentifier, ResourceConsumption> userResourceConsumption3 =
+  private static final String HOSTNAME3 = "host3";
+  private static final int RPCPORT3 = 3111;
+  private static final int PUSHPORT3 = 3112;
+  private static final int FETCHPORT3 = 3113;
+  private static final int REPLICATEPORT3 = 3114;
+  private static final Map<String, DiskInfo> disks3 = new HashMap<>();
+  private static final Map<UserIdentifier, ResourceConsumption> userResourceConsumption3 =
       new HashMap<>();
 
   @Before
-  public void setUp() throws Exception {
+  public void setUp() {
     when(mockRpcEnv.setupEndpointRef(any(), any())).thenReturn(dummyRef);
     statusSystem = new SingleMasterMetaManager(mockRpcEnv, conf);
 
@@ -102,7 +103,7 @@ public class DefaultMetaSystemSuiteJ {
   }
 
   @After
-  public void tearDown() throws Exception {}
+  public void tearDown() {}
 
   private String getNewReqeustId() {
     return MasterClient.encodeRequestId(UUID.randomUUID().toString(), callerId.incrementAndGet());
@@ -139,7 +140,56 @@ public class DefaultMetaSystemSuiteJ {
         userResourceConsumption3,
         getNewReqeustId());
 
-    assert (statusSystem.workers.size() == 3);
+    assertEquals(3, statusSystem.workers.size());
+  }
+
+  @Test
+  public void testHandleWorkerExclude() {
+    WorkerInfo workerInfo1 =
+        new WorkerInfo(
+            HOSTNAME1,
+            RPCPORT1,
+            PUSHPORT1,
+            FETCHPORT1,
+            REPLICATEPORT1,
+            disks1,
+            userResourceConsumption1);
+    WorkerInfo workerInfo2 =
+        new WorkerInfo(
+            HOSTNAME2,
+            RPCPORT2,
+            PUSHPORT2,
+            FETCHPORT2,
+            REPLICATEPORT2,
+            disks2,
+            userResourceConsumption2);
+
+    statusSystem.handleRegisterWorker(
+        workerInfo1.host(),
+        workerInfo1.rpcPort(),
+        workerInfo1.pushPort(),
+        workerInfo1.fetchPort(),
+        workerInfo1.replicatePort(),
+        workerInfo1.diskInfos(),
+        workerInfo1.userResourceConsumption(),
+        getNewReqeustId());
+    statusSystem.handleRegisterWorker(
+        workerInfo2.host(),
+        workerInfo2.rpcPort(),
+        workerInfo2.pushPort(),
+        workerInfo2.fetchPort(),
+        workerInfo2.replicatePort(),
+        workerInfo2.diskInfos(),
+        workerInfo2.userResourceConsumption(),
+        getNewReqeustId());
+
+    statusSystem.handleWorkerExclude(
+        Arrays.asList(workerInfo1, workerInfo2), Collections.emptyList(), getNewReqeustId());
+    assertEquals(2, statusSystem.manuallyExcludedWorkers.size());
+
+    statusSystem.handleWorkerExclude(
+        Collections.emptyList(), Collections.singletonList(workerInfo1), getNewReqeustId());
+    assertEquals(1, statusSystem.manuallyExcludedWorkers.size());
   }
 
   @Test
@@ -174,12 +224,12 @@ public class DefaultMetaSystemSuiteJ {
 
     statusSystem.handleWorkerLost(
         HOSTNAME1, RPCPORT1, PUSHPORT1, FETCHPORT1, REPLICATEPORT1, getNewReqeustId());
-    assert (statusSystem.workers.size() == 2);
+    assertEquals(2, statusSystem.workers.size());
   }
 
-  private static String APPID1 = "appId1";
-  private static int SHUFFLEID1 = 1;
-  private static String SHUFFLEKEY1 = APPID1 + "-" + SHUFFLEID1;
+  private static final String APPID1 = "appId1";
+  private static final int SHUFFLEID1 = 1;
+  private static final String SHUFFLEKEY1 = APPID1 + "-" + SHUFFLEID1;
 
   @Test
   public void testHandleRequestSlots() {
@@ -247,9 +297,9 @@ public class DefaultMetaSystemSuiteJ {
 
     statusSystem.handleRequestSlots(SHUFFLEKEY1, HOSTNAME1, workersToAllocate, getNewReqeustId());
 
-    assert (workerInfo1.usedSlots() == 0);
-    assert (workerInfo2.usedSlots() == 0);
-    assert (workerInfo3.usedSlots() == 0);
+    assertEquals(0, workerInfo1.usedSlots());
+    assertEquals(0, workerInfo2.usedSlots());
+    assertEquals(0, workerInfo3.usedSlots());
   }
 
   @Test
@@ -282,7 +332,7 @@ public class DefaultMetaSystemSuiteJ {
         userResourceConsumption3,
         getNewReqeustId());
 
-    assert 3 == statusSystem.workers.size();
+    assertEquals(3, statusSystem.workers.size());
 
     Map<String, Map<String, Integer>> workersToAllocate = new HashMap<>();
     Map<String, Integer> allocation = new HashMap<>();
@@ -303,20 +353,7 @@ public class DefaultMetaSystemSuiteJ {
         allocation);
 
     statusSystem.handleRequestSlots(SHUFFLEKEY1, HOSTNAME1, workersToAllocate, getNewReqeustId());
-
-    List<String> workerIds = new ArrayList<>();
-    workerIds.add(
-        HOSTNAME1 + ":" + RPCPORT1 + ":" + PUSHPORT1 + ":" + FETCHPORT1 + ":" + REPLICATEPORT1);
-
-    List<Map<String, Integer>> workerSlots = new ArrayList<>();
-    workerSlots.add(
-        new HashMap() {
-          {
-            put("disk1", 3);
-          }
-        });
-
-    Assert.assertEquals(
+    assertEquals(
         0,
         statusSystem.workers.stream()
             .filter(w -> w.host().equals(HOSTNAME1))
@@ -382,11 +419,11 @@ public class DefaultMetaSystemSuiteJ {
 
     statusSystem.handleRequestSlots(SHUFFLEKEY1, HOSTNAME1, workersToAllocate, getNewReqeustId());
 
-    assert statusSystem.registeredShuffle.size() == 1;
+    assertEquals(1, statusSystem.registeredShuffle.size());
 
     statusSystem.handleAppLost(APPID1, getNewReqeustId());
 
-    assert statusSystem.registeredShuffle.isEmpty();
+    assertTrue(statusSystem.registeredShuffle.isEmpty());
   }
 
   @Test
@@ -446,24 +483,24 @@ public class DefaultMetaSystemSuiteJ {
 
     statusSystem.handleRequestSlots(SHUFFLEKEY1, HOSTNAME1, workersToAllocate, getNewReqeustId());
 
-    assert statusSystem.registeredShuffle.size() == 1;
+    assertEquals(1, statusSystem.registeredShuffle.size());
 
     statusSystem.handleUnRegisterShuffle(SHUFFLEKEY1, getNewReqeustId());
 
-    assert statusSystem.registeredShuffle.isEmpty();
+    assertTrue(statusSystem.registeredShuffle.isEmpty());
   }
 
   @Test
   public void testHandleAppHeartbeat() {
-    long dummy = 1235L;
+    Long dummy = 1235L;
     statusSystem.handleAppHeartbeat(APPID1, 1, 1, dummy, getNewReqeustId());
-    assert statusSystem.appHeartbeatTime.get(APPID1) == dummy;
+    assertEquals(dummy, statusSystem.appHeartbeatTime.get(APPID1));
 
     String appId2 = "app02";
     statusSystem.handleAppHeartbeat(appId2, 1, 1, dummy, getNewReqeustId());
-    assert statusSystem.appHeartbeatTime.get(appId2) == dummy;
+    assertEquals(dummy, statusSystem.appHeartbeatTime.get(appId2));
 
-    assert statusSystem.appHeartbeatTime.size() == 2;
+    assertEquals(2, statusSystem.appHeartbeatTime.size());
   }
 
   @Test
@@ -509,7 +546,7 @@ public class DefaultMetaSystemSuiteJ {
         false,
         getNewReqeustId());
 
-    Assert.assertEquals(statusSystem.excludedWorkers.size(), 1);
+    assertEquals(statusSystem.excludedWorkers.size(), 1);
 
     statusSystem.handleWorkerHeartbeat(
         HOSTNAME2,
@@ -524,7 +561,7 @@ public class DefaultMetaSystemSuiteJ {
         false,
         getNewReqeustId());
 
-    Assert.assertEquals(statusSystem.excludedWorkers.size(), 2);
+    assertEquals(statusSystem.excludedWorkers.size(), 2);
 
     statusSystem.handleWorkerHeartbeat(
         HOSTNAME3,
@@ -539,7 +576,7 @@ public class DefaultMetaSystemSuiteJ {
         false,
         getNewReqeustId());
 
-    Assert.assertEquals(statusSystem.excludedWorkers.size(), 2);
+    assertEquals(statusSystem.excludedWorkers.size(), 2);
 
     statusSystem.handleWorkerHeartbeat(
         HOSTNAME3,
@@ -554,7 +591,7 @@ public class DefaultMetaSystemSuiteJ {
         true,
         getNewReqeustId());
 
-    Assert.assertEquals(statusSystem.excludedWorkers.size(), 3);
+    assertEquals(statusSystem.excludedWorkers.size(), 3);
   }
 
   @Test
@@ -587,7 +624,8 @@ public class DefaultMetaSystemSuiteJ {
         userResourceConsumption3,
         getNewReqeustId());
 
-    WorkerInfo workerInfo1 =
+    List<WorkerInfo> failedWorkers = new ArrayList<>();
+    failedWorkers.add(
         new WorkerInfo(
             HOSTNAME1,
             RPCPORT1,
@@ -595,23 +633,11 @@ public class DefaultMetaSystemSuiteJ {
             FETCHPORT1,
             REPLICATEPORT1,
             disks1,
-            userResourceConsumption1);
-    WorkerInfo workerInfo2 =
-        new WorkerInfo(
-            HOSTNAME2,
-            RPCPORT2,
-            PUSHPORT2,
-            FETCHPORT2,
-            REPLICATEPORT2,
-            disks2,
-            userResourceConsumption2);
-
-    List<WorkerInfo> failedWorkers = new ArrayList<>();
-    failedWorkers.add(workerInfo1);
+            userResourceConsumption1));
 
     statusSystem.handleReportWorkerUnavailable(failedWorkers, getNewReqeustId());
-    assert 1 == statusSystem.shutdownWorkers.size();
-    assert 0 == statusSystem.excludedWorkers.size();
+    assertEquals(1, statusSystem.shutdownWorkers.size());
+    assertTrue(statusSystem.excludedWorkers.isEmpty());
   }
 
   @Test
