@@ -47,7 +47,7 @@ class ShuffleClientSuite extends WithShuffleClientSuite with MiniClusterFeature
 
     val lifecycleManager: LifecycleManager = new LifecycleManager(APP, celebornConf)
     val shuffleClient: ShuffleClientImpl = {
-      val client = new ShuffleClientImpl(APP, celebornConf, userIdentifier, false)
+      val client = new ShuffleClientImpl(APP, celebornConf, userIdentifier)
       client.setupLifecycleManagerRef(lifecycleManager.self)
       client
     }
