@@ -33,6 +33,7 @@ import org.slf4j.LoggerFactory;
 
 import org.apache.celeborn.common.meta.FileInfo;
 import org.apache.celeborn.common.util.JavaUtils;
+import org.apache.celeborn.common.util.ThreadUtils;
 import org.apache.celeborn.service.deploy.worker.memory.MemoryManager;
 
 public class CreditStreamManager {
@@ -48,28 +49,28 @@ public class CreditStreamManager {
   private final BlockingQueue<DelayedStreamId> recycleStreamIds = new DelayQueue<>();
 
   @GuardedBy("lock")
-  private volatile Thread recycleThread;
+  private volatile ExecutorService recycleThread;
 
   private final Object lock = new Object();
 
   public CreditStreamManager(
       int minReadBuffers,
       int maxReadBuffers,
-      int threadsPerMountpoint,
+      int threadsPerMountPoint,
       int minBuffersToTriggerRead) {
     nextStreamId = new AtomicLong((long) new Random().nextInt(Integer.MAX_VALUE) * 1000);
     streams = JavaUtils.newConcurrentHashMap();
     activeMapPartitions = JavaUtils.newConcurrentHashMap();
     this.minReadBuffers = minReadBuffers;
     this.maxReadBuffers = maxReadBuffers;
-    threadsPerMountPoint = threadsPerMountpoint;
+    this.threadsPerMountPoint = threadsPerMountPoint;
     this.minBuffersToTriggerRead = minBuffersToTriggerRead;
     MemoryManager.instance().setCreditStreamManager(this);
     logger.debug(
         "Initialize buffer stream manager with {} {} {}",
         this.minReadBuffers,
         this.maxReadBuffers,
-        threadsPerMountpoint);
+        threadsPerMountPoint);
   }
 
   public long registerStream(
@@ -171,11 +172,6 @@ public class CreditStreamManager {
     startRecycleThread(); // lazy start thread
   }
 
-  @VisibleForTesting
-  public int numStreamStates() {
-    return streams.size();
-  }
-
   @VisibleForTesting
   public int numRecycleStreams() {
     return recycleStreamIds.size();
@@ -187,27 +183,23 @@ public class CreditStreamManager {
   }
 
   private void startRecycleThread() {
-    if (recycleThread == null) {
-      synchronized (lock) {
-        if (recycleThread == null) {
-          recycleThread =
-              new Thread(
-                  () -> {
-                    while (true) {
-                      try {
-                        DelayedStreamId delayedStreamId = recycleStreamIds.take();
-                        cleanResource(delayedStreamId.streamId);
-                      } catch (Throwable e) {
-                        logger.warn(e.getMessage(), e);
-                      }
-                    }
-                  },
-                  "recycle-thread");
-          recycleThread.setDaemon(true);
-          recycleThread.start();
+    synchronized (lock) {
+      if (recycleThread == null) {
+        recycleThread =
+            ThreadUtils.newDaemonSingleThreadExecutor("worker-credit-stream-manager-recycler");
+        recycleThread.submit(
+            () -> {
+              while (true) {
+                try {
+                  DelayedStreamId delayedStreamId = recycleStreamIds.take();
+                  cleanResource(delayedStreamId.streamId);
+                } catch (Throwable e) {
+                  logger.warn(e.getMessage(), e);
+                }
+              }
+            });
 
-          logger.info("start stream recycle thread");
-        }
+        logger.info("start stream recycle thread");
       }
     }
   }
