@@ -23,23 +23,21 @@ import java.util
 import java.util.concurrent.atomic.AtomicBoolean
 import java.util.function.Consumer
 
-import scala.collection.JavaConverters.asScalaBufferConverter
-
 import com.google.common.base.Throwables
 import com.google.protobuf.GeneratedMessageV3
 import io.netty.util.concurrent.{Future, GenericFutureListener}
 
 import org.apache.celeborn.common.CelebornConf
 import org.apache.celeborn.common.CelebornConf.MAX_CHUNKS_BEING_TRANSFERRED
+import org.apache.celeborn.common.exception.CelebornIOException
 import org.apache.celeborn.common.internal.Logging
 import org.apache.celeborn.common.meta.{FileInfo, FileManagedBuffers}
 import org.apache.celeborn.common.network.buffer.NioManagedBuffer
-import org.apache.celeborn.common.network.client.TransportClient
+import org.apache.celeborn.common.network.client.{RpcResponseCallback, TransportClient}
 import org.apache.celeborn.common.network.protocol._
-import org.apache.celeborn.common.network.protocol.Message.Type
 import org.apache.celeborn.common.network.server.BaseMessageHandler
 import org.apache.celeborn.common.network.util.{NettyUtils, TransportConf}
-import org.apache.celeborn.common.protocol.{MessageType, PartitionType, PbBufferStreamEnd, PbOpenStream, PbReadAddCredit, PbStreamHandler}
+import org.apache.celeborn.common.protocol.{MessageType, PartitionType, PbBufferStreamEnd, PbChunkFetchRequest, PbOpenStream, PbReadAddCredit, PbStreamHandler, StreamType}
 import org.apache.celeborn.common.util.{ExceptionUtils, Utils}
 import org.apache.celeborn.service.deploy.worker.storage.{ChunkStreamManager, CreditStreamManager, PartitionFilesSorter, StorageManager}
 
@@ -62,6 +60,9 @@ class FetchHandler(
   var registered: AtomicBoolean = new AtomicBoolean(false)
 
   def init(worker: Worker): Unit = {
+    workerSource.addGauge(WorkerSource.CHUNK_STREAM_COUNT) { () =>
+      chunkStreamManager.getStreamsCount
+    }
 
     workerSource.addGauge(WorkerSource.CREDIT_STREAM_COUNT) { () =>
       creditStreamManager.getStreamsCount
@@ -89,6 +90,13 @@ class FetchHandler(
     fileInfo
   }
 
+  override def receive(
+      client: TransportClient,
+      msg: RequestMessage,
+      callback: RpcResponseCallback): Unit = {
+    handleRpcRequest(client, msg.asInstanceOf[RpcRequest], callback)
+  }
+
   override def receive(client: TransportClient, msg: RequestMessage): Unit = {
     msg match {
       case r: BufferStreamEnd =>
@@ -96,97 +104,96 @@ class FetchHandler(
       case r: ReadAddCredit =>
         handleReadAddCredit(r.getCredit, r.getStreamId)
       case r: ChunkFetchRequest =>
-        handleChunkFetchRequest(client, r)
-      case r: RpcRequest =>
-        var streamShuffleKey: String = null
-        try {
-          val pbMsg = TransportMessage.fromByteBuffer(
-            r.body().nioByteBuffer()).getParsedPayload.asInstanceOf[GeneratedMessageV3]
-          pbMsg match {
-            case pb: PbBufferStreamEnd => handleEndStreamFromClient(pb.getStreamId)
-            case pb: PbReadAddCredit => handleReadAddCredit(pb.getCredit, pb.getStreamId)
-            case pb: PbOpenStream =>
-              val (shuffleKey, fileName, startIndex, endIndex, initialCredit, readLocalShuffle) =
-                (
-                  pb.getShuffleKey,
-                  pb.getFileName,
-                  pb.getStartIndex,
-                  pb.getEndIndex,
-                  pb.getInitialCredit,
-                  pb.getReadLocalShuffle)
-              streamShuffleKey = shuffleKey
-              workerSource.startTimer(WorkerSource.OPEN_STREAM_TIME, streamShuffleKey)
-              handleOpenStreamInternal(
-                client,
-                shuffleKey,
-                fileName,
-                startIndex,
-                endIndex,
-                initialCredit,
-                r,
-                false,
-                readLocalShuffle)
-          }
-        } catch {
-          case _: Exception =>
-            logDebug("Legacy RPCs")
-            val decodedMsg = Message.decode(r.body().nioByteBuffer())
-            val msgType = decodedMsg.`type`()
-            if (msgType == Type.OPEN_STREAM || msgType == Type.OPEN_STREAM_WITH_CREDIT) {
-              var streamFileName: String = null
-              try {
-                val (shuffleKey, fileName) =
-                  if (msgType == Type.OPEN_STREAM) {
-                    val openStream = decodedMsg.asInstanceOf[OpenStream]
-                    (
-                      new String(openStream.shuffleKey, StandardCharsets.UTF_8),
-                      new String(openStream.fileName, StandardCharsets.UTF_8))
-                  } else {
-                    val openStreamWithCredit = decodedMsg.asInstanceOf[OpenStreamWithCredit]
-                    (
-                      new String(openStreamWithCredit.shuffleKey, StandardCharsets.UTF_8),
-                      new String(openStreamWithCredit.fileName, StandardCharsets.UTF_8))
-                  }
-                streamShuffleKey = shuffleKey
-                streamFileName = fileName
-                var startIndex = 0
-                var endIndex = 0
-                var initialCredit = 0
-                getRawFileInfo(shuffleKey, fileName).getPartitionType match {
-                  case PartitionType.REDUCE =>
-                    startIndex = decodedMsg.asInstanceOf[OpenStream].startMapIndex
-                    endIndex = decodedMsg.asInstanceOf[OpenStream].endMapIndex
-                  case PartitionType.MAP =>
-                    initialCredit = decodedMsg.asInstanceOf[OpenStreamWithCredit].initialCredit
-                    startIndex = decodedMsg.asInstanceOf[OpenStreamWithCredit].startIndex
-                    endIndex = decodedMsg.asInstanceOf[OpenStreamWithCredit].endIndex
-                  case PartitionType.MAPGROUP =>
-                }
-                handleOpenStreamInternal(
-                  client,
-                  shuffleKey,
-                  fileName,
-                  startIndex,
-                  endIndex,
-                  initialCredit,
-                  r,
-                  true)
-              } catch {
-                case e: IOException =>
-                  handleRpcIOException(client, r.requestId, streamShuffleKey, streamFileName, e)
-              }
-            }
-        } finally {
-          r.body().release()
-          if (streamShuffleKey != null) {
-            workerSource.stopTimer(WorkerSource.OPEN_STREAM_TIME, streamShuffleKey)
-          }
-        }
+        handleChunkFetchRequest(client, r.streamChunkSlice, r)
       case unknown: RequestMessage =>
         throw new IllegalArgumentException(s"Unknown message type id: ${unknown.`type`.id}")
     }
   }
 
+  private def handleRpcRequest(
+      client: TransportClient,
+      rpcRequest: RpcRequest,
+      callback: RpcResponseCallback): Unit = {
+    var message: GeneratedMessageV3 = null
+    try {
+      message = TransportMessage.fromByteBuffer(rpcRequest.body().nioByteBuffer())
+        .getParsedPayload[GeneratedMessageV3]
+    } catch {
+      case exception: CelebornIOException =>
+        logWarning("Handle request with legacy RPCs", exception)
+        return handleLegacyRpcMessage(client, rpcRequest, callback)
+    }
+    message match {
+      case openStream: PbOpenStream =>
+        handleOpenStreamInternal(
+          client,
+          openStream.getShuffleKey,
+          openStream.getFileName,
+          openStream.getStartIndex,
+          openStream.getEndIndex,
+          openStream.getInitialCredit,
+          rpcRequest.requestId,
+          isLegacy = false,
+          openStream.getReadLocalShuffle,
+          callback)
+      case bufferStreamEnd: PbBufferStreamEnd =>
+        handleEndStreamFromClient(bufferStreamEnd.getStreamId, bufferStreamEnd.getStreamType)
+      case readAddCredit: PbReadAddCredit =>
+        handleReadAddCredit(readAddCredit.getCredit, readAddCredit.getStreamId)
+      case chunkFetchRequest: PbChunkFetchRequest =>
+        handleChunkFetchRequest(
+          client,
+          StreamChunkSlice.fromProto(chunkFetchRequest.getStreamChunkSlice),
+          rpcRequest)
+      case message: GeneratedMessageV3 =>
+        logError(s"Unknown message $message")
+    }
+  }
+
+  private def handleLegacyRpcMessage(
+      client: TransportClient,
+      rpcRequest: RpcRequest,
+      callback: RpcResponseCallback): Unit = {
+    try {
+      val message = Message.decode(rpcRequest.body().nioByteBuffer())
+      message.`type`() match {
+        case Message.Type.OPEN_STREAM =>
+          val openStream = message.asInstanceOf[OpenStream]
+          handleOpenStreamInternal(
+            client,
+            new String(openStream.shuffleKey, StandardCharsets.UTF_8),
+            new String(openStream.fileName, StandardCharsets.UTF_8),
+            openStream.startMapIndex,
+            openStream.endMapIndex,
+            initialCredit = 0,
+            rpcRequestId = rpcRequest.requestId,
+            isLegacy = true,
+            // legacy [[OpenStream]] doesn't support read local shuffle
+            readLocalShuffle = false,
+            callback)
+        case Message.Type.OPEN_STREAM_WITH_CREDIT =>
+          val openStreamWithCredit = message.asInstanceOf[OpenStreamWithCredit]
+          handleOpenStreamInternal(
+            client,
+            new String(openStreamWithCredit.shuffleKey, StandardCharsets.UTF_8),
+            new String(openStreamWithCredit.fileName, StandardCharsets.UTF_8),
+            openStreamWithCredit.startIndex,
+            openStreamWithCredit.endIndex,
+            openStreamWithCredit.initialCredit,
+            rpcRequestId = rpcRequest.requestId,
+            isLegacy = true,
+            readLocalShuffle = false,
+            callback)
+        case _ =>
+          logError(s"Received an unknown message type id: ${message.`type`.id}")
+      }
+    } catch {
+      case e: Exception =>
+        logError("Catch an error when handle legacy rpc message.", e)
+    }
+
+  }
+
   private def handleOpenStreamInternal(
       client: TransportClient,
       shuffleKey: String,
@@ -194,14 +201,26 @@ class FetchHandler(
       startIndex: Int,
       endIndex: Int,
       initialCredit: Int,
-      request: RpcRequest,
+      rpcRequestId: Long,
       isLegacy: Boolean,
-      readLocalShuffle: Boolean = false): Unit = {
+      readLocalShuffle: Boolean = false,
+      callback: RpcResponseCallback): Unit = {
+    workerSource.startTimer(WorkerSource.OPEN_STREAM_TIME, shuffleKey)
     try {
       var fileInfo = getRawFileInfo(shuffleKey, fileName)
       fileInfo.getPartitionType match {
         case PartitionType.REDUCE =>
-          if (endIndex != Integer.MAX_VALUE) {
+          logDebug(s"Received open stream request $shuffleKey $fileName $startIndex " +
+            s"$endIndex get file name $fileName from client channel " +
+            s"${NettyUtils.getRemoteAddress(client.getChannel)}")
+
+          val streamId = chunkStreamManager.nextStreamId()
+          // we must get sorted fileInfo for the following cases.
+          // 1. when the current request is a non-range openStream, but the original unsorted file
+          //    has been deleted by another range's openStream request.
+          // 2. when the current request is a range openStream request.
+          if ((endIndex != Int.MaxValue) || (endIndex == Int.MaxValue && !fileInfo.addStream(
+              streamId))) {
             fileInfo = partitionsSorter.getSortedFileInfo(
               shuffleKey,
               fileName,
@@ -209,42 +228,47 @@ class FetchHandler(
               startIndex,
               endIndex)
           }
-          logDebug(s"Received chunk fetch request $shuffleKey $fileName $startIndex " +
-            s"$endIndex get file info $fileInfo from client channel " +
-            s"${NettyUtils.getRemoteAddress(client.getChannel)}")
           if (readLocalShuffle) {
+            chunkStreamManager.registerStream(
+              streamId,
+              shuffleKey,
+              fileName)
             replyStreamHandler(
               client,
-              request.requestId,
-              -1,
+              rpcRequestId,
+              streamId,
               fileInfo.numChunks(),
               isLegacy,
               fileInfo.getChunkOffsets,
               fileInfo.getFilePath)
           } else if (fileInfo.isHdfs) {
-            replyStreamHandler(client, request.requestId, 0, 0, isLegacy)
-          } else {
-            val buffers = new FileManagedBuffers(fileInfo, transportConf)
-            val fetchTimeMetrics = storageManager.getFetchTimeMetric(fileInfo.getFile)
-            val streamId = chunkStreamManager.registerStream(
+            chunkStreamManager.registerStream(
+              streamId,
               shuffleKey,
-              buffers,
-              fetchTimeMetrics)
-            if (fileInfo.numChunks() == 0)
+              fileName)
+            replyStreamHandler(client, rpcRequestId, streamId, numChunks = 0, isLegacy)
+          } else {
+            chunkStreamManager.registerStream(
+              streamId,
+              shuffleKey,
+              new FileManagedBuffers(fileInfo, transportConf),
+              fileName,
+              storageManager.getFetchTimeMetric(fileInfo.getFile))
+            if (fileInfo.numChunks == 0)
               logDebug(s"StreamId $streamId, fileName $fileName, mapRange " +
                 s"[$startIndex-$endIndex] is empty. Received from client channel " +
                 s"${NettyUtils.getRemoteAddress(client.getChannel)}")
             else logDebug(
-              s"StreamId $streamId, fileName $fileName, numChunks ${fileInfo.numChunks()}, " +
+              s"StreamId $streamId, fileName $fileName, numChunks ${fileInfo.numChunks}, " +
                 s"mapRange [$startIndex-$endIndex]. Received from client channel " +
                 s"${NettyUtils.getRemoteAddress(client.getChannel)}")
-            replyStreamHandler(client, request.requestId, streamId, fileInfo.numChunks(), isLegacy)
+            replyStreamHandler(client, rpcRequestId, streamId, fileInfo.numChunks(), isLegacy)
           }
         case PartitionType.MAP =>
           val creditStreamHandler =
             new Consumer[java.lang.Long] {
               override def accept(streamId: java.lang.Long): Unit = {
-                replyStreamHandler(client, request.requestId, streamId, 0, isLegacy)
+                replyStreamHandler(client, rpcRequestId, streamId, 0, isLegacy)
               }
             }
 
@@ -257,9 +281,13 @@ class FetchHandler(
             fileInfo)
         case PartitionType.MAPGROUP =>
       }
+      workerSource.incCounter(WorkerSource.OPEN_STREAM_SUCCESS_COUNT)
     } catch {
       case e: IOException =>
-        handleRpcIOException(client, request.requestId, shuffleKey, fileName, e)
+        workerSource.incCounter(WorkerSource.OPEN_STREAM_FAIL_COUNT)
+        handleRpcIOException(client, rpcRequestId, shuffleKey, fileName, e, callback)
+    } finally {
+      workerSource.stopTimer(WorkerSource.OPEN_STREAM_TIME, shuffleKey)
     }
   }
 
@@ -298,36 +326,51 @@ class FetchHandler(
       requestId: Long,
       shuffleKey: String,
       fileName: String,
-      ioe: IOException): Unit = {
+      ioe: IOException,
+      rpcCallback: RpcResponseCallback): Unit = {
     // if open stream rpc failed, this IOException actually should be FileNotFoundException
     // we wrapper this IOException(Other place may have other exception like FileCorruptException) unify to
     // PartitionUnRetryableException for reader can give up this partition and choose to regenerate the partition data
     logError(
       s"Read file: $fileName with shuffleKey: $shuffleKey error from ${NettyUtils.getRemoteAddress(client.getChannel)}",
       ioe)
-    handleRpcException(client, requestId, ioe)
+    handleRpcException(client, requestId, ioe, rpcCallback)
   }
 
   private def handleRpcException(
       client: TransportClient,
       requestId: Long,
-      ioe: IOException): Unit = {
-    client.getChannel.writeAndFlush(new RpcFailure(
-      requestId,
-      Throwables.getStackTraceAsString(ExceptionUtils.wrapIOExceptionToUnRetryable(ioe))))
+      ioe: IOException,
+      rpcResponseCallback: RpcResponseCallback): Unit = {
+    rpcResponseCallback.onFailure(ExceptionUtils.wrapIOExceptionToUnRetryable(ioe))
   }
 
   def handleEndStreamFromClient(streamId: Long): Unit = {
-    creditStreamManager.notifyStreamEndByClient(streamId)
+    handleEndStreamFromClient(streamId, StreamType.CreditStream)
+  }
+
+  def handleEndStreamFromClient(streamId: Long, streamType: StreamType): Unit = {
+    streamType match {
+      case StreamType.ChunkStream =>
+        val (shuffleKey, fileName) = chunkStreamManager.getShuffleKeyAndFileName(streamId)
+        getRawFileInfo(shuffleKey, fileName).closeStream(streamId)
+      case StreamType.CreditStream =>
+        creditStreamManager.notifyStreamEndByClient(streamId)
+      case _ =>
+        logError(s"Received a PbBufferStreamEnd message with unknown type $streamType")
+    }
   }
 
   def handleReadAddCredit(credit: Int, streamId: Long): Unit = {
     creditStreamManager.addCredit(credit, streamId)
   }
 
-  def handleChunkFetchRequest(client: TransportClient, req: ChunkFetchRequest): Unit = {
+  def handleChunkFetchRequest(
+      client: TransportClient,
+      streamChunkSlice: StreamChunkSlice,
+      req: RequestMessage): Unit = {
     logDebug(s"Received req from ${NettyUtils.getRemoteAddress(client.getChannel)}" +
-      s" to fetch block ${req.streamChunkSlice}")
+      s" to fetch block $streamChunkSlice")
 
     maxChunkBeingTransferred.foreach { threshold =>
       val chunksBeingTransferred = chunkStreamManager.chunksBeingTransferred // take high cpu usage
@@ -336,51 +379,55 @@ class FetchHandler(
           s"$chunksBeingTransferred exceeds ${MAX_CHUNKS_BEING_TRANSFERRED.key} " +
           s"${Utils.bytesToString(threshold)}."
         logError(message)
-        client.getChannel.writeAndFlush(new ChunkFetchFailure(req.streamChunkSlice, message))
+        workerSource.incCounter(WorkerSource.FETCH_CHUNK_FAIL_COUNT)
+        client.getChannel.writeAndFlush(new ChunkFetchFailure(streamChunkSlice, message))
         return
       }
     }
 
-    workerSource.startTimer(WorkerSource.FETCH_CHUNK_TIME, req.toString)
-    val fetchTimeMetric = chunkStreamManager.getFetchTimeMetric(req.streamChunkSlice.streamId)
+    val reqStr = req.toString
+    workerSource.startTimer(WorkerSource.FETCH_CHUNK_TIME, reqStr)
+    val fetchTimeMetric = chunkStreamManager.getFetchTimeMetric(streamChunkSlice.streamId)
     val fetchBeginTime = System.nanoTime()
     try {
       val buf = chunkStreamManager.getChunk(
-        req.streamChunkSlice.streamId,
-        req.streamChunkSlice.chunkIndex,
-        req.streamChunkSlice.offset,
-        req.streamChunkSlice.len)
-      chunkStreamManager.chunkBeingSent(req.streamChunkSlice.streamId)
-      client.getChannel.writeAndFlush(new ChunkFetchSuccess(req.streamChunkSlice, buf))
+        streamChunkSlice.streamId,
+        streamChunkSlice.chunkIndex,
+        streamChunkSlice.offset,
+        streamChunkSlice.len)
+      chunkStreamManager.chunkBeingSent(streamChunkSlice.streamId)
+      client.getChannel.writeAndFlush(new ChunkFetchSuccess(streamChunkSlice, buf))
         .addListener(new GenericFutureListener[Future[_ >: Void]] {
           override def operationComplete(future: Future[_ >: Void]): Unit = {
-            if (future.isSuccess()) {
+            if (future.isSuccess) {
               if (log.isDebugEnabled) {
                 logDebug(
-                  s"Sending ChunkFetchSuccess operation succeeded, chunk ${req.streamChunkSlice}")
+                  s"Sending ChunkFetchSuccess operation succeeded, chunk $streamChunkSlice")
               }
             } else {
-              logError(
-                s"Sending ChunkFetchSuccess operation failed, chunk ${req.streamChunkSlice}",
+              logWarning(
+                s"Sending ChunkFetchSuccess operation failed, chunk $streamChunkSlice",
                 future.cause())
             }
-            chunkStreamManager.chunkSent(req.streamChunkSlice.streamId)
+            workerSource.incCounter(WorkerSource.FETCH_CHUNK_SUCCESS_COUNT)
+            chunkStreamManager.chunkSent(streamChunkSlice.streamId)
             if (fetchTimeMetric != null) {
               fetchTimeMetric.update(System.nanoTime() - fetchBeginTime)
             }
-            workerSource.stopTimer(WorkerSource.FETCH_CHUNK_TIME, req.toString)
+            workerSource.stopTimer(WorkerSource.FETCH_CHUNK_TIME, reqStr)
           }
         })
     } catch {
       case e: Exception =>
         logError(
-          s"Error opening block ${req.streamChunkSlice} for request from " +
+          s"Error opening block $streamChunkSlice for request from " +
             NettyUtils.getRemoteAddress(client.getChannel),
           e)
+        workerSource.incCounter(WorkerSource.FETCH_CHUNK_FAIL_COUNT)
         client.getChannel.writeAndFlush(new ChunkFetchFailure(
-          req.streamChunkSlice,
+          streamChunkSlice,
           Throwables.getStackTraceAsString(e)))
-        workerSource.stopTimer(WorkerSource.FETCH_CHUNK_TIME, req.toString)
+        workerSource.stopTimer(WorkerSource.FETCH_CHUNK_TIME, reqStr)
     }
   }
 
@@ -405,5 +452,13 @@ class FetchHandler(
 
   def cleanupExpiredShuffleKey(expiredShuffleKeys: util.HashSet[String]): Unit = {
     chunkStreamManager.cleanupExpiredShuffleKey(expiredShuffleKeys)
+    // maybe null when running unit test.
+    if (partitionsSorter != null) {
+      partitionsSorter.cleanupExpiredShuffleKey(expiredShuffleKeys)
+    }
+  }
+
+  def setPartitionsSorter(partitionFilesSorter: PartitionFilesSorter): Unit = {
+    this.partitionsSorter = partitionFilesSorter
   }
 }
