@@ -21,6 +21,11 @@ import java.nio.ByteBuffer
 import java.util.concurrent.{ConcurrentHashMap, ThreadPoolExecutor}
 import java.util.concurrent.atomic.{AtomicBoolean, AtomicIntegerArray}
 
+import scala.collection.JavaConverters._
+import scala.concurrent.{Await, Promise}
+import scala.concurrent.duration.Duration
+import scala.util.{Failure, Success, Try}
+
 import com.google.common.base.Throwables
 import com.google.protobuf.GeneratedMessageV3
 import io.netty.buffer.ByteBuf
@@ -50,11 +55,12 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
   private var shufflePushDataTimeout: ConcurrentHashMap[String, Long] = _
   private var replicateThreadPool: ThreadPoolExecutor = _
   private var unavailablePeers: ConcurrentHashMap[WorkerInfo, Long] = _
-  private var pushClientFactory: TransportClientFactory = _
+  private var replicateClientFactory: TransportClientFactory = _
   private var registered: AtomicBoolean = _
   private var workerInfo: WorkerInfo = _
   private var diskReserveSize: Long = _
   private var diskReserveRatio: Option[Double] = _
+  private var diskUsableSizes: Map[String, Long] = _
   private var partitionSplitMinimumSize: Long = _
   private var partitionSplitMaximumSize: Long = _
   private var shutdown: AtomicBoolean = _
@@ -72,11 +78,14 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
     shuffleMapperAttempts = worker.shuffleMapperAttempts
     replicateThreadPool = worker.replicateThreadPool
     unavailablePeers = worker.unavailablePeers
-    pushClientFactory = worker.pushClientFactory
+    replicateClientFactory = worker.replicateClientFactory
     registered = worker.registered
     workerInfo = worker.workerInfo
     diskReserveSize = worker.conf.workerDiskReserveSize
     diskReserveRatio = worker.conf.workerDiskReserveRatio
+    diskUsableSizes = workerInfo.diskInfos.asScala.map { case (mountPoint, diskInfo) =>
+      (mountPoint, DiskUtils.getMinimumUsableSize(diskInfo, diskReserveSize, diskReserveRatio))
+    }.toMap
     partitionSplitMinimumSize = worker.conf.partitionSplitMinimumSize
     partitionSplitMaximumSize = worker.conf.partitionSplitMaximumSize
     storageManager = worker.storageManager
@@ -91,18 +100,25 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
       s"diskReserveSize ${Utils.bytesToString(diskReserveSize)}, diskReserveRatio ${diskReserveRatio.orNull}")
   }
 
+  override def receive(
+      client: TransportClient,
+      msg: RequestMessage,
+      callback: RpcResponseCallback): Unit = {
+    handleRpcRequest(client, msg.asInstanceOf[RpcRequest], callback)
+  }
+
   override def receive(client: TransportClient, msg: RequestMessage): Unit =
     msg match {
       case pushData: PushData =>
+        val callback = new SimpleRpcResponseCallback(
+          client,
+          pushData.requestId,
+          pushData.shuffleKey)
         handleCore(
           client,
           pushData,
           pushData.requestId,
           () => {
-            val callback = new SimpleRpcResponseCallback(
-              client,
-              pushData.requestId,
-              pushData.shuffleKey)
             val partitionType =
               shufflePartitionType.getOrDefault(pushData.shuffleKey, PartitionType.REDUCE)
             partitionType match {
@@ -114,8 +130,13 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
                   callback)
               case _ => throw new UnsupportedOperationException(s"Not support $partitionType yet")
             }
-          })
+          },
+          callback)
       case pushMergedData: PushMergedData =>
+        val callback = new SimpleRpcResponseCallback(
+          client,
+          pushMergedData.requestId,
+          pushMergedData.shuffleKey)
         handleCore(
           client,
           pushMergedData,
@@ -123,11 +144,8 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
           () =>
             handlePushMergedData(
               pushMergedData,
-              new SimpleRpcResponseCallback(
-                client,
-                pushMergedData.requestId,
-                pushMergedData.shuffleKey)))
-      case rpcRequest: RpcRequest => handleRpcRequest(client, rpcRequest)
+              callback),
+          callback)
     }
 
   def handlePushData(pushData: PushData, callback: RpcResponseCallback): Unit = {
@@ -184,14 +202,15 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
       if (shuffleMapperAttempts.containsKey(shuffleKey)) {
         if (-1 != shuffleMapperAttempts.get(shuffleKey).get(mapId)) {
           // partition data has already been committed
-          logInfo(
+          logDebug(
             s"[Case1] Receive push data from speculative task(shuffle $shuffleKey, map $mapId, " +
               s" attempt $attemptId), but this mapper has already been ended.")
           callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte](StatusCode.MAP_ENDED.getValue)))
         } else {
-          logInfo(
+          logDebug(
             s"Receive push data for committed hard split partition of (shuffle $shuffleKey, " +
               s"map $mapId attempt $attemptId)")
+          workerSource.incCounter(WorkerSource.WRITE_DATA_HARD_SPLIT_COUNT)
           callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte](StatusCode.HARD_SPLIT.getValue)))
         }
       } else {
@@ -199,7 +218,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
           // If there is no shuffle key in shuffleMapperAttempts but there is shuffle key
           // in StorageManager. This partition should be HARD_SPLIT partition and
           // after worker restart, some tasks still push data to this HARD_SPLIT partition.
-          logInfo(s"[Case2] Receive push data for committed hard split partition of " +
+          logDebug(s"[Case2] Receive push data for committed hard split partition of " +
             s"(shuffle $shuffleKey, map $mapId attempt $attemptId)")
           callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte](StatusCode.HARD_SPLIT.getValue)))
         } else {
@@ -248,7 +267,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
       fileWriter.decrementPendingWrites()
       return;
     }
-
+    val writePromise = Promise[Unit]()
     // for primary, send data to replica
     if (doReplicate) {
       pushData.body().retain()
@@ -274,34 +293,38 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
           // Handle the response from replica
           val wrappedCallback = new RpcResponseCallback() {
             override def onSuccess(response: ByteBuffer): Unit = {
-              if (response.remaining() > 0) {
-                val resp = ByteBuffer.allocate(response.remaining())
-                resp.put(response)
-                resp.flip()
-                callbackWithTimer.onSuccess(resp)
-              } else if (softSplit.get()) {
-                // TODO Currently if the worker is in soft split status, given the guess that the client
-                // will fast stop pushing data to the worker, we won't return congest status. But
-                // in the long term, especially if this issue could frequently happen, we may need to return
-                // congest&softSplit status together
-                callbackWithTimer.onSuccess(
-                  ByteBuffer.wrap(Array[Byte](StatusCode.SOFT_SPLIT.getValue)))
-              } else {
-                Option(CongestionController.instance()) match {
-                  case Some(congestionController) =>
-                    if (congestionController.isUserCongested(
-                        fileWriter.getFileInfo.getUserIdentifier)) {
-                      // Check whether primary congest the data though the replicas doesn't congest
-                      // it(the response is empty)
-                      callbackWithTimer.onSuccess(
-                        ByteBuffer.wrap(
-                          Array[Byte](StatusCode.PUSH_DATA_SUCCESS_PRIMARY_CONGESTED.getValue)))
-                    } else {
-                      callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
+              Try(Await.result(writePromise.future, Duration.Inf)) match {
+                case Success(_) =>
+                  if (response.remaining() > 0) {
+                    val resp = ByteBuffer.allocate(response.remaining())
+                    resp.put(response)
+                    resp.flip()
+                    callbackWithTimer.onSuccess(resp)
+                  } else if (softSplit.get()) {
+                    // TODO Currently if the worker is in soft split status, given the guess that the client
+                    // will fast stop pushing data to the worker, we won't return congest status. But
+                    // in the long term, especially if this issue could frequently happen, we may need to return
+                    // congest&softSplit status together
+                    callbackWithTimer.onSuccess(
+                      ByteBuffer.wrap(Array[Byte](StatusCode.SOFT_SPLIT.getValue)))
+                  } else {
+                    Option(CongestionController.instance()) match {
+                      case Some(congestionController) =>
+                        if (congestionController.isUserCongested(
+                            fileWriter.getFileInfo.getUserIdentifier)) {
+                          // Check whether primary congest the data though the replicas doesn't congest
+                          // it(the response is empty)
+                          callbackWithTimer.onSuccess(
+                            ByteBuffer.wrap(
+                              Array[Byte](StatusCode.PUSH_DATA_SUCCESS_PRIMARY_CONGESTED.getValue)))
+                        } else {
+                          callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
+                        }
+                      case None =>
+                        callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
                     }
-                  case None =>
-                    callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
-                }
+                  }
+                case Failure(e) => callbackWithTimer.onFailure(e)
               }
             }
 
@@ -324,7 +347,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
             }
           }
           try {
-            val client = getClient(peer.getHost, peer.getReplicatePort, location.getId)
+            val client = getReplicateClient(peer.getHost, peer.getReplicatePort, location.getId)
             val newPushData = new PushData(
               PartitionLocation.Mode.REPLICA.mode(),
               shuffleKey,
@@ -344,6 +367,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
           }
         }
       })
+      writeLocalData(Seq(fileWriter), body, shuffleKey, isPrimary, None, writePromise)
     } else {
       // The codes here could be executed if
       // 1. the client doesn't enable push data to the replica, the primary worker could hit here
@@ -352,47 +376,36 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
       // will fast stop pushing data to the worker, we won't return congest status. But
       // in the long term, especially if this issue could frequently happen, we may need to return
       // congest&softSplit status together
-      if (softSplit.get()) {
-        callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte](StatusCode.SOFT_SPLIT.getValue)))
-      } else {
-        Option(CongestionController.instance()) match {
-          case Some(congestionController) =>
-            if (congestionController.isUserCongested(fileWriter.getFileInfo.getUserIdentifier)) {
-              if (isPrimary) {
-                callbackWithTimer.onSuccess(
-                  ByteBuffer.wrap(
-                    Array[Byte](StatusCode.PUSH_DATA_SUCCESS_PRIMARY_CONGESTED.getValue)))
-              } else {
-                callbackWithTimer.onSuccess(
-                  ByteBuffer.wrap(
-                    Array[Byte](StatusCode.PUSH_DATA_SUCCESS_REPLICA_CONGESTED.getValue)))
-              }
-            } else {
-              callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
+      writeLocalData(Seq(fileWriter), body, shuffleKey, isPrimary, None, writePromise)
+      Try(Await.result(writePromise.future, Duration.Inf)) match {
+        case Success(_) =>
+          if (softSplit.get()) {
+            callbackWithTimer.onSuccess(
+              ByteBuffer.wrap(Array[Byte](StatusCode.SOFT_SPLIT.getValue)))
+          } else {
+            Option(CongestionController.instance()) match {
+              case Some(congestionController) =>
+                if (congestionController.isUserCongested(
+                    fileWriter.getFileInfo.getUserIdentifier)) {
+                  if (isPrimary) {
+                    callbackWithTimer.onSuccess(
+                      ByteBuffer.wrap(
+                        Array[Byte](StatusCode.PUSH_DATA_SUCCESS_PRIMARY_CONGESTED.getValue)))
+                  } else {
+                    callbackWithTimer.onSuccess(
+                      ByteBuffer.wrap(
+                        Array[Byte](StatusCode.PUSH_DATA_SUCCESS_REPLICA_CONGESTED.getValue)))
+                  }
+                } else {
+                  callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
+                }
+              case None =>
+                callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
             }
-          case None =>
-            callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
-        }
+          }
+        case Failure(e) => callbackWithTimer.onFailure(e)
       }
     }
-
-    try {
-      fileWriter.write(body)
-    } catch {
-      case e: AlreadyClosedException =>
-        fileWriter.decrementPendingWrites()
-        val (mapId, attemptId) = getMapAttempt(body)
-        val endedAttempt =
-          if (shuffleMapperAttempts.containsKey(shuffleKey)) {
-            shuffleMapperAttempts.get(shuffleKey).get(mapId)
-          } else -1
-        // TODO just info log for ended attempt
-        logError(
-          s"[handlePushData] Append data failed for task(shuffle $shuffleKey, map $mapId, attempt" +
-            s" $attemptId), caused by AlreadyClosedException, endedAttempt $endedAttempt, error message: ${e.getMessage}")
-      case e: Exception =>
-        logError("Exception encountered when write.", e)
-    }
   }
 
   def handlePushMergedData(
@@ -454,14 +467,15 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
         // it's probably because commitFiles for HARD_SPLIT happens.
         if (shuffleMapperAttempts.containsKey(shuffleKey)) {
           if (-1 != shuffleMapperAttempts.get(shuffleKey).get(mapId)) {
-            logInfo(s"Receive push merged data from speculative " +
+            logDebug(s"Receive push merged data from speculative " +
               s"task(shuffle $shuffleKey, map $mapId, attempt $attemptId), " +
               s"but this mapper has already been ended.")
             callbackWithTimer.onSuccess(
               ByteBuffer.wrap(Array[Byte](StatusCode.MAP_ENDED.getValue)))
           } else {
-            logInfo(s"[Case1] Receive push merged data for committed hard split partition of " +
+            logDebug(s"[Case1] Receive push merged data for committed hard split partition of " +
               s"(shuffle $shuffleKey, map $mapId attempt $attemptId)")
+            workerSource.incCounter(WorkerSource.WRITE_DATA_HARD_SPLIT_COUNT)
             callbackWithTimer.onSuccess(
               ByteBuffer.wrap(Array[Byte](StatusCode.HARD_SPLIT.getValue)))
           }
@@ -470,8 +484,9 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
             // If there is no shuffle key in shuffleMapperAttempts but there is shuffle key
             // in StorageManager. This partition should be HARD_SPLIT partition and
             // after worker restart, some tasks still push data to this HARD_SPLIT partition.
-            logInfo(s"[Case2] Receive push merged data for committed hard split partition of " +
+            logDebug(s"[Case2] Receive push merged data for committed hard split partition of " +
               s"(shuffle $shuffleKey, map $mapId attempt $attemptId)")
+            workerSource.incCounter(WorkerSource.WRITE_DATA_HARD_SPLIT_COUNT)
             callbackWithTimer.onSuccess(
               ByteBuffer.wrap(Array[Byte](StatusCode.HARD_SPLIT.getValue)))
           } else {
@@ -519,7 +534,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
       fileWriters.foreach(_.decrementPendingWrites())
       return
     }
-
+    val writePromise = Promise[Unit]()
     // for primary, send data to replica
     if (doReplicate) {
       pushMergedData.body().retain()
@@ -546,28 +561,32 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
           // Handle the response from replica
           val wrappedCallback = new RpcResponseCallback() {
             override def onSuccess(response: ByteBuffer): Unit = {
-              // Only primary data enable replication will push data to replica
-              if (response.remaining() > 0) {
-                val resp = ByteBuffer.allocate(response.remaining())
-                resp.put(response)
-                resp.flip()
-                callbackWithTimer.onSuccess(resp)
-              } else {
-                Option(CongestionController.instance()) match {
-                  case Some(congestionController) if fileWriters.nonEmpty =>
-                    if (congestionController.isUserCongested(
-                        fileWriters.head.getFileInfo.getUserIdentifier)) {
-                      // Check whether primary congest the data though the replicas doesn't congest
-                      // it(the response is empty)
-                      callbackWithTimer.onSuccess(
-                        ByteBuffer.wrap(
-                          Array[Byte](StatusCode.PUSH_DATA_SUCCESS_PRIMARY_CONGESTED.getValue)))
-                    } else {
-                      callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
+              Try(Await.result(writePromise.future, Duration.Inf)) match {
+                case Success(_) =>
+                  // Only primary data enable replication will push data to replica
+                  if (response.remaining() > 0) {
+                    val resp = ByteBuffer.allocate(response.remaining())
+                    resp.put(response)
+                    resp.flip()
+                    callbackWithTimer.onSuccess(resp)
+                  } else {
+                    Option(CongestionController.instance()) match {
+                      case Some(congestionController) if fileWriters.nonEmpty =>
+                        if (congestionController.isUserCongested(
+                            fileWriters.head.getFileInfo.getUserIdentifier)) {
+                          // Check whether primary congest the data though the replicas doesn't congest
+                          // it(the response is empty)
+                          callbackWithTimer.onSuccess(
+                            ByteBuffer.wrap(
+                              Array[Byte](StatusCode.PUSH_DATA_SUCCESS_PRIMARY_CONGESTED.getValue)))
+                        } else {
+                          callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
+                        }
+                      case None =>
+                        callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
                     }
-                  case None =>
-                    callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
-                }
+                  }
+                case Failure(e) => callbackWithTimer.onFailure(e)
               }
             }
 
@@ -591,7 +610,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
           }
 
           try {
-            val client = getClient(peer.getHost, peer.getReplicatePort, location.getId)
+            val client = getReplicateClient(peer.getHost, peer.getReplicatePort, location.getId)
             val newPushMergedData = new PushMergedData(
               PartitionLocation.Mode.REPLICA.mode(),
               shuffleKey,
@@ -615,69 +634,36 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
           }
         }
       })
+      writeLocalData(fileWriters, body, shuffleKey, isPrimary, Some(batchOffsets), writePromise)
     } else {
       // The codes here could be executed if
       // 1. the client doesn't enable push data to the replica, the primary worker could hit here
       // 2. the client enables push data to the replica, and the replica worker could hit here
-      Option(CongestionController.instance()) match {
-        case Some(congestionController) if fileWriters.nonEmpty =>
-          if (congestionController.isUserCongested(
-              fileWriters.head.getFileInfo.getUserIdentifier)) {
-            if (isPrimary) {
-              callbackWithTimer.onSuccess(
-                ByteBuffer.wrap(
-                  Array[Byte](StatusCode.PUSH_DATA_SUCCESS_PRIMARY_CONGESTED.getValue)))
-            } else {
-              callbackWithTimer.onSuccess(
-                ByteBuffer.wrap(
-                  Array[Byte](StatusCode.PUSH_DATA_SUCCESS_REPLICA_CONGESTED.getValue)))
-            }
-          } else {
-            callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
+      writeLocalData(fileWriters, body, shuffleKey, isPrimary, Some(batchOffsets), writePromise)
+      Try(Await.result(writePromise.future, Duration.Inf)) match {
+        case Success(_) =>
+          Option(CongestionController.instance()) match {
+            case Some(congestionController) if fileWriters.nonEmpty =>
+              if (congestionController.isUserCongested(
+                  fileWriters.head.getFileInfo.getUserIdentifier)) {
+                if (isPrimary) {
+                  callbackWithTimer.onSuccess(
+                    ByteBuffer.wrap(
+                      Array[Byte](StatusCode.PUSH_DATA_SUCCESS_PRIMARY_CONGESTED.getValue)))
+                } else {
+                  callbackWithTimer.onSuccess(
+                    ByteBuffer.wrap(
+                      Array[Byte](StatusCode.PUSH_DATA_SUCCESS_REPLICA_CONGESTED.getValue)))
+                }
+              } else {
+                callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
+              }
+            case None =>
+              callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
           }
-        case None =>
-          callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte]()))
+        case Failure(e) => callbackWithTimer.onFailure(e)
       }
     }
-
-    index = 0
-    var fileWriter: FileWriter = null
-    var alreadyClosed = false
-    while (index < fileWriters.length) {
-      fileWriter = fileWriters(index)
-      val offset = body.readerIndex() + batchOffsets(index)
-      val length =
-        if (index == fileWriters.length - 1) {
-          body.readableBytes() - batchOffsets(index)
-        } else {
-          batchOffsets(index + 1) - batchOffsets(index)
-        }
-      val batchBody = body.slice(offset, length)
-
-      try {
-        if (!alreadyClosed) {
-          fileWriter.write(batchBody)
-        } else {
-          fileWriter.decrementPendingWrites()
-        }
-      } catch {
-        case e: AlreadyClosedException =>
-          fileWriter.decrementPendingWrites()
-          alreadyClosed = true
-          val (mapId, attemptId) = getMapAttempt(body)
-          val endedAttempt =
-            if (shuffleMapperAttempts.containsKey(shuffleKey)) {
-              shuffleMapperAttempts.get(shuffleKey).get(mapId)
-            } else -1
-          // TODO just info log for ended attempt
-          logError(
-            s"[handlePushMergedData] Append data failed for task(shuffle $shuffleKey, map $mapId, attempt" +
-              s" $attemptId), caused by AlreadyClosedException, endedAttempt $endedAttempt, error message: ${e.getMessage}")
-        case e: Exception =>
-          logError("Exception encountered when write.", e)
-      }
-      index += 1
-    }
   }
 
   /**
@@ -752,17 +738,14 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
       client: TransportClient,
       message: RequestMessage,
       requestId: Long,
-      handler: () => Unit): Unit = {
+      handler: () => Unit,
+      callback: RpcResponseCallback): Unit = {
     try {
       handler()
     } catch {
       case e: Exception =>
         logError(s"Error while handle${message.`type`()} $message", e)
-        client.getChannel.writeAndFlush(new RpcFailure(
-          requestId,
-          Throwables.getStackTraceAsString(e)));
-    } finally {
-      message.body().release()
+        callback.onFailure(e)
     }
   }
 
@@ -821,35 +804,27 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
       fileWriter.decrementPendingWrites()
       return;
     }
-
+    val writePromise = Promise[Unit]()
+    writeLocalData(Seq(fileWriter), body, shuffleKey, isPrimary, None, writePromise)
     // for primary, send data to replica
     if (location.hasPeer && isPrimary) {
       // to do
-      wrappedCallback.onSuccess(ByteBuffer.wrap(Array[Byte]()))
+      Try(Await.result(writePromise.future, Duration.Inf)) match {
+        case Success(_) => wrappedCallback.onSuccess(ByteBuffer.wrap(Array[Byte]()))
+        case Failure(e) => wrappedCallback.onFailure(e)
+      }
     } else {
-      wrappedCallback.onSuccess(ByteBuffer.wrap(Array[Byte]()))
-    }
-
-    try {
-      fileWriter.write(body)
-    } catch {
-      case e: AlreadyClosedException =>
-        fileWriter.decrementPendingWrites()
-        val (mapId, attemptId) = getMapAttempt(body)
-        val endedAttempt =
-          if (shuffleMapperAttempts.containsKey(shuffleKey)) {
-            shuffleMapperAttempts.get(shuffleKey).get(mapId)
-          } else -1
-        // TODO just info log for ended attempt
-        logError(
-          s"[handleMapPartitionPushData] Append data failed for task(shuffle $shuffleKey, map $mapId, attempt" +
-            s" $attemptId), caused by AlreadyClosedException, endedAttempt $endedAttempt, error message: ${e.getMessage}")
-      case e: Exception =>
-        logError("Exception encountered when write.", e)
+      Try(Await.result(writePromise.future, Duration.Inf)) match {
+        case Success(_) => wrappedCallback.onSuccess(ByteBuffer.wrap(Array[Byte]()))
+        case Failure(e) => wrappedCallback.onFailure(e)
+      }
     }
   }
 
-  private def handleRpcRequest(client: TransportClient, rpcRequest: RpcRequest): Unit = {
+  private def handleRpcRequest(
+      client: TransportClient,
+      rpcRequest: RpcRequest,
+      callback: RpcResponseCallback): Unit = {
     val requestId = rpcRequest.requestId
     val (pbMsg, msg, isLegacy, messageType, mode, shuffleKey, partitionUniqueId, checkSplit) =
       mapPartitionRpcRequest(rpcRequest)
@@ -871,7 +846,8 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
           new SimpleRpcResponseCallback(
             client,
             requestId,
-            shuffleKey)))
+            shuffleKey)),
+      callback)
   }
 
   private def mapPartitionRpcRequest(rpcRequest: RpcRequest)
@@ -1044,7 +1020,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
             numPartitions,
             bufferSize)
         case Type.REGION_START =>
-          val (currentRegionIndex, isBroadcast) =
+          val (currentRegionIndex, isBroadcast: Boolean) =
             if (isLegacy)
               (
                 msg.asInstanceOf[RegionStart].currentRegionIndex,
@@ -1052,7 +1028,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
             else
               (
                 pbMsg.asInstanceOf[PbRegionStart].getCurrentRegionIndex,
-                Boolean.box(pbMsg.asInstanceOf[PbRegionStart].getIsBroadcast))
+                pbMsg.asInstanceOf[PbRegionStart].getIsBroadcast)
           fileWriter.asInstanceOf[MapPartitionFileWriter].regionStart(
             currentRegionIndex,
             isBroadcast)
@@ -1195,14 +1171,10 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
     if (fileWriter.flusher.isInstanceOf[HdfsFlusher]) {
       return false
     }
-    val diskInfo = workerInfo.diskInfos
-      .get(fileWriter.flusher.asInstanceOf[LocalFlusher].mountPoint)
-
-    val minimumUsableSize =
-      DiskUtils.getMinimumUsableSize(diskInfo, diskReserveSize, diskReserveRatio)
+    val mountPoint = fileWriter.flusher.asInstanceOf[LocalFlusher].mountPoint
+    val diskInfo = workerInfo.diskInfos.get(mountPoint)
     val diskFull = diskInfo.status.equals(
-      DiskStatus.HIGH_DISK_USAGE) || diskInfo.actualUsableSpace < minimumUsableSize
-
+      DiskStatus.HIGH_DISK_USAGE) || diskInfo.actualUsableSpace < diskUsableSizes(mountPoint)
     diskFull
   }
 
@@ -1227,6 +1199,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
         (fileWriter.getFileInfo.getFileLength < partitionSplitMaximumSize)) {
         softSplit.set(true)
       } else {
+        workerSource.incCounter(WorkerSource.WRITE_DATA_HARD_SPLIT_COUNT)
         callback.onSuccess(ByteBuffer.wrap(Array[Byte](StatusCode.HARD_SPLIT.getValue)))
         logTrace(
           s"""
@@ -1243,11 +1216,76 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
     false
   }
 
-  private def getClient(host: String, port: Int, partitionId: Int): TransportClient = {
+  private def getReplicateClient(host: String, port: Int, partitionId: Int): TransportClient = {
     if (workerReplicateRandomConnectionEnabled) {
-      pushClientFactory.createClient(host, port)
+      replicateClientFactory.createClient(host, port)
     } else {
-      pushClientFactory.createClient(host, port, partitionId)
+      replicateClientFactory.createClient(host, port, partitionId)
+    }
+  }
+
+  private def writeLocalData(
+      fileWriters: Seq[FileWriter],
+      body: ByteBuf,
+      shuffleKey: String,
+      isPrimary: Boolean,
+      batchOffsets: Option[Array[Int]],
+      writePromise: Promise[Unit]): Unit = {
+    def writeData(fileWriter: FileWriter, body: ByteBuf, shuffleKey: String): Unit = {
+      try {
+        fileWriter.write(body)
+      } catch {
+        case e: Exception =>
+          if (e.isInstanceOf[AlreadyClosedException]) {
+            val (mapId, attemptId) = getMapAttempt(body)
+            val endedAttempt =
+              if (shuffleMapperAttempts.containsKey(shuffleKey)) {
+                shuffleMapperAttempts.get(shuffleKey).get(mapId)
+              } else -1
+            // TODO just info log for ended attempt
+            logWarning(s"Append data failed for task(shuffle $shuffleKey, map $mapId, attempt" +
+              s" $attemptId), caused by AlreadyClosedException, endedAttempt $endedAttempt, error message: ${e.getMessage}")
+          } else {
+            logError("Exception encountered when write.", e)
+          }
+          val cause =
+            if (isPrimary) {
+              StatusCode.PUSH_DATA_WRITE_FAIL_PRIMARY
+            } else {
+              StatusCode.PUSH_DATA_WRITE_FAIL_REPLICA
+            }
+          workerSource.incCounter(WorkerSource.WRITE_DATA_FAIL_COUNT)
+          writePromise.failure(new CelebornIOException(cause))
+          fileWriter.decrementPendingWrites()
+      }
+    }
+    batchOffsets match {
+      case Some(batchOffsets) =>
+        var index = 0
+        var fileWriter: FileWriter = null
+        while (index < fileWriters.length) {
+          if (!writePromise.isCompleted) {
+            fileWriter = fileWriters(index)
+            val offset = body.readerIndex() + batchOffsets(index)
+            val length =
+              if (index == fileWriters.length - 1) {
+                body.readableBytes() - batchOffsets(index)
+              } else {
+                batchOffsets(index + 1) - batchOffsets(index)
+              }
+            val batchBody = body.slice(offset, length)
+            writeData(fileWriter, batchBody, shuffleKey)
+          } else {
+            fileWriter.decrementPendingWrites()
+          }
+          index += 1
+        }
+      case _ =>
+        writeData(fileWriters.head, body, shuffleKey)
+    }
+    if (!writePromise.isCompleted) {
+      workerSource.incCounter(WorkerSource.WRITE_DATA_SUCCESS_COUNT)
+      writePromise.success()
     }
   }
 
