@@ -19,7 +19,8 @@ package org.apache.celeborn.service.deploy.worker
 
 import java.io.File
 import java.lang.{Long => JLong}
-import java.util.{HashMap => JHashMap, HashSet => JHashSet, Map => JMap}
+import java.util
+import java.util.{HashMap => JHashMap, HashSet => JHashSet, Locale, Map => JMap}
 import java.util.concurrent._
 import java.util.concurrent.atomic.{AtomicBoolean, AtomicIntegerArray}
 
@@ -36,7 +37,7 @@ import org.apache.celeborn.common.identity.UserIdentifier
 import org.apache.celeborn.common.internal.Logging
 import org.apache.celeborn.common.meta.{DiskInfo, WorkerInfo, WorkerPartitionLocationInfo}
 import org.apache.celeborn.common.metrics.MetricsSystem
-import org.apache.celeborn.common.metrics.source.{JVMCPUSource, JVMSource, SystemMiscSource}
+import org.apache.celeborn.common.metrics.source.{JVMCPUSource, JVMSource, ResourceConsumptionSource, SystemMiscSource, ThreadPoolSource}
 import org.apache.celeborn.common.network.TransportContext
 import org.apache.celeborn.common.protocol.{PartitionType, PbRegisterWorkerResponse, PbWorkerLostResponse, RpcNameConstants, TransportModuleConstants}
 import org.apache.celeborn.common.protocol.message.ControlMessages._
@@ -50,6 +51,7 @@ import org.apache.celeborn.service.deploy.worker.WorkerSource.ACTIVE_CONNECTION_
 import org.apache.celeborn.service.deploy.worker.congestcontrol.CongestionController
 import org.apache.celeborn.service.deploy.worker.memory.{ChannelsLimiter, MemoryManager}
 import org.apache.celeborn.service.deploy.worker.memory.MemoryManager.ServingState
+import org.apache.celeborn.service.deploy.worker.monitor.JVMQuake
 import org.apache.celeborn.service.deploy.worker.storage.{PartitionFilesSorter, StorageManager}
 
 private[celeborn] class Worker(
@@ -62,7 +64,17 @@ private[celeborn] class Worker(
   override def serviceName: String = Service.WORKER
 
   override val metricsSystem: MetricsSystem =
-    MetricsSystem.createMetricsSystem(serviceName, conf, MetricsSystem.SERVLET_PATH)
+    MetricsSystem.createMetricsSystem(serviceName, conf)
+  val workerSource = new WorkerSource(conf)
+  private val resourceConsumptionSource =
+    new ResourceConsumptionSource(conf, MetricsSystem.ROLE_WORKER)
+  private val threadPoolSource = ThreadPoolSource(conf, MetricsSystem.ROLE_WORKER)
+  metricsSystem.registerSource(workerSource)
+  metricsSystem.registerSource(threadPoolSource)
+  metricsSystem.registerSource(resourceConsumptionSource)
+  metricsSystem.registerSource(new JVMSource(conf, MetricsSystem.ROLE_WORKER))
+  metricsSystem.registerSource(new JVMCPUSource(conf, MetricsSystem.ROLE_WORKER))
+  metricsSystem.registerSource(new SystemMiscSource(conf, MetricsSystem.ROLE_WORKER))
 
   val rpcEnv: RpcEnv = RpcEnv.create(
     RpcNameConstants.WORKER_SYS,
@@ -104,12 +116,6 @@ private[celeborn] class Worker(
     }
   }
 
-  val workerSource = new WorkerSource(conf)
-  metricsSystem.registerSource(workerSource)
-  metricsSystem.registerSource(new JVMSource(conf, MetricsSystem.ROLE_WORKER))
-  metricsSystem.registerSource(new JVMCPUSource(conf, MetricsSystem.ROLE_WORKER))
-  metricsSystem.registerSource(new SystemMiscSource(conf, MetricsSystem.ROLE_WORKER))
-
   val storageManager = new StorageManager(conf, workerSource)
 
   val memoryManager: MemoryManager = MemoryManager.initialize(conf)
@@ -136,7 +142,7 @@ private[celeborn] class Worker(
   rpcEnv.setupEndpoint(RpcNameConstants.WORKER_EP, controller)
 
   val pushDataHandler = new PushDataHandler(workerSource)
-  val (pushServer, pushClientFactory) = {
+  private val pushServer = {
     val closeIdleConnections = conf.workerCloseIdleConnections
     val numThreads = conf.workerPushIoThreads.getOrElse(storageManager.totalFlusherThread)
     val transportConf =
@@ -150,13 +156,11 @@ private[celeborn] class Worker(
         pushServerLimiter,
         conf.workerPushHeartbeatEnabled,
         workerSource)
-    (
-      transportContext.createServer(conf.workerPushPort),
-      transportContext.createClientFactory())
+    transportContext.createServer(conf.workerPushPort)
   }
 
   val replicateHandler = new PushDataHandler(workerSource)
-  private val replicateServer = {
+  val (replicateServer, replicateClientFactory) = {
     val closeIdleConnections = conf.workerCloseIdleConnections
     val numThreads =
       conf.workerReplicateIoThreads.getOrElse(storageManager.totalFlusherThread)
@@ -171,7 +175,9 @@ private[celeborn] class Worker(
         replicateLimiter,
         false,
         workerSource)
-    transportContext.createServer(conf.workerReplicatePort)
+    (
+      transportContext.createServer(conf.workerReplicatePort),
+      transportContext.createClientFactory())
   }
 
   var fetchHandler: FetchHandler = _
@@ -208,11 +214,6 @@ private[celeborn] class Worker(
     diskInfos.put(diskInfo.mountPoint, diskInfo)
   }
 
-  // need to ensure storageManager has recovered fileinfos data if enable graceful shutdown before retrieve consumption
-  val userResourceConsumption: ConcurrentHashMap[UserIdentifier, ResourceConsumption] =
-    JavaUtils.newConcurrentHashMap[UserIdentifier, ResourceConsumption](
-      storageManager.userResourceConsumptionSnapshot().asJava)
-
   val workerInfo =
     new WorkerInfo(
       host,
@@ -221,7 +222,7 @@ private[celeborn] class Worker(
       fetchPort,
       replicatePort,
       diskInfos,
-      userResourceConsumption)
+      JavaUtils.newConcurrentHashMap[UserIdentifier, ResourceConsumption])
 
   // whether this Worker registered to Master successfully
   val registered = new AtomicBoolean(false)
@@ -249,15 +250,15 @@ private[celeborn] class Worker(
   private var checkFastFailTask: ScheduledFuture[_] = _
 
   val replicateThreadPool: ThreadPoolExecutor =
-    ThreadUtils.newDaemonCachedThreadPool("worker-replicate-data", conf.workerReplicateThreads)
+    ThreadUtils.newDaemonCachedThreadPool("worker-data-replicator", conf.workerReplicateThreads)
   val commitThreadPool: ThreadPoolExecutor =
-    ThreadUtils.newDaemonCachedThreadPool("worker-commit-files", conf.workerCommitThreads)
+    ThreadUtils.newDaemonCachedThreadPool("worker-files-committer", conf.workerCommitThreads)
   val cleanThreadPool: ThreadPoolExecutor =
     ThreadUtils.newDaemonCachedThreadPool(
-      "worker-clean-expired-shuffle-keys",
+      "worker-expired-shuffle-cleaner",
       conf.workerCleanThreads)
   val asyncReplyPool: ScheduledExecutorService =
-    ThreadUtils.newDaemonSingleThreadScheduledExecutor("async-reply")
+    ThreadUtils.newDaemonSingleThreadScheduledExecutor("worker-rpc-async-replier")
   val timer = new HashedWheelTimer()
 
   // Configs
@@ -265,11 +266,21 @@ private[celeborn] class Worker(
   private val replicaFastFailDuration = conf.workerReplicateFastFailDuration
 
   private val cleanTaskQueue = new LinkedBlockingQueue[JHashSet[String]]
-  var cleaner: Thread = _
+  var cleaner: ExecutorService =
+    ThreadUtils.newDaemonSingleThreadExecutor("worker-expired-shuffle-cleaner")
+
+  private var jvmQuake: JVMQuake = _
+  if (conf.workerJvmQuakeEnabled) {
+    jvmQuake = JVMQuake.create(conf, workerInfo.toUniqueId().replace(":", "-"))
+    jvmQuake.start()
+  }
 
   workerSource.addGauge(WorkerSource.REGISTERED_SHUFFLE_COUNT) { () =>
     workerInfo.getShuffleKeySet.size
   }
+  workerSource.addGauge(WorkerSource.RUNNING_APPLICATION_COUNT) { () =>
+    workerInfo.getApplicationIdSet.size
+  }
   workerSource.addGauge(WorkerSource.SORT_MEMORY) { () =>
     memoryManager.getSortMemoryCounter.get()
   }
@@ -288,12 +299,6 @@ private[celeborn] class Worker(
   workerSource.addGauge(WorkerSource.NETTY_MEMORY) { () =>
     memoryManager.getNettyUsedDirectMemory
   }
-  workerSource.addGauge(WorkerSource.PAUSE_PUSH_DATA_COUNT) { () =>
-    memoryManager.getPausePushDataCounter
-  }
-  workerSource.addGauge(WorkerSource.PAUSE_PUSH_DATA_AND_REPLICATE_COUNT) { () =>
-    memoryManager.getPausePushDataAndReplicateCounter
-  }
   workerSource.addGauge(WorkerSource.BUFFER_STREAM_READ_BUFFER) { () =>
     memoryManager.getReadBufferCounter
   }
@@ -312,6 +317,15 @@ private[celeborn] class Worker(
   workerSource.addGauge(WorkerSource.PAUSE_PUSH_DATA_TIME) { () =>
     memoryManager.getPausePushDataTime
   }
+  workerSource.addGauge(WorkerSource.PAUSE_PUSH_DATA_AND_REPLICATE_TIME) { () =>
+    memoryManager.getPausePushDataAndReplicateTime
+  }
+  workerSource.addGauge(WorkerSource.PAUSE_PUSH_DATA_COUNT) { () =>
+    memoryManager.getPausePushDataCounter
+  }
+  workerSource.addGauge(WorkerSource.PAUSE_PUSH_DATA_AND_REPLICATE_COUNT) { () =>
+    memoryManager.getPausePushDataAndReplicateCounter
+  }
 
   private def highWorkload: Boolean = {
     (memoryManager.currentServingState, conf.workerActiveConnectionMax) match {
@@ -335,10 +349,7 @@ private[celeborn] class Worker(
     val diskInfos =
       workerInfo.updateThenGetDiskInfos(storageManager.disksSnapshot().map { disk =>
         disk.mountPoint -> disk
-      }.toMap.asJava).values().asScala.toSeq
-    val resourceConsumption = workerInfo.updateThenGetUserResourceConsumption(
-      storageManager.userResourceConsumptionSnapshot().asJava)
-
+      }.toMap.asJava).values().asScala.toSeq ++ storageManager.hdfsDiskInfo
     val response = masterClient.askSync[HeartbeatFromWorkerResponse](
       HeartbeatFromWorker(
         host,
@@ -347,7 +358,7 @@ private[celeborn] class Worker(
         fetchPort,
         replicatePort,
         diskInfos,
-        resourceConsumption,
+        handleResourceConsumption(),
         activeShuffleKeys,
         estimatedAppDiskUsage,
         highWorkload),
@@ -396,7 +407,7 @@ private[celeborn] class Worker(
       replicaFastFailDuration,
       TimeUnit.MILLISECONDS)
 
-    cleaner = new Thread("Cleaner") {
+    cleaner.submit(new Runnable {
       override def run(): Unit = {
         while (true) {
           val expiredShuffleKeys = cleanTaskQueue.take()
@@ -408,16 +419,13 @@ private[celeborn] class Worker(
           }
         }
       }
-    }
+    })
 
     pushDataHandler.init(this)
     replicateHandler.init(this)
     fetchHandler.init(this)
     controller.init(this)
 
-    cleaner.setDaemon(true)
-    cleaner.start()
-
     logInfo("Worker started.")
     rpcEnv.awaitTermination()
   }
@@ -426,6 +434,9 @@ private[celeborn] class Worker(
     if (!stopped) {
       logInfo("Stopping Worker.")
 
+      if (jvmQuake != null) {
+        jvmQuake.stop()
+      }
       if (sendHeartbeatTask != null) {
         if (exitKind == CelebornExitKind.WORKER_GRACEFUL_SHUTDOWN) {
           sendHeartbeatTask.cancel(false)
@@ -488,8 +499,7 @@ private[celeborn] class Worker(
               // Use WorkerInfo's diskInfo since re-register when heartbeat return not-registered,
               // StorageManager have update the disk info.
               workerInfo.diskInfos.asScala.toMap,
-              workerInfo.updateThenGetUserResourceConsumption(
-                storageManager.userResourceConsumptionSnapshot().asJava).asScala.toMap,
+              handleResourceConsumption().asScala.toMap,
               MasterClient.genRequestId()),
             classOf[PbRegisterWorkerResponse])
         } catch {
@@ -513,6 +523,34 @@ private[celeborn] class Worker(
     // If worker register still failed after retry, throw exception to stop worker process
     throw new CelebornException("Register worker failed.", exception)
   }
+
+  private def handleResourceConsumption(): util.Map[UserIdentifier, ResourceConsumption] = {
+    val resourceConsumptionSnapshot = storageManager.userResourceConsumptionSnapshot()
+    resourceConsumptionSnapshot.foreach { case (userIdentifier, _) =>
+      resourceConsumptionSource.addGauge(
+        ResourceConsumptionSource.DISK_FILE_COUNT,
+        userIdentifier.toMap) { () =>
+        workerInfo.userResourceConsumption.get(userIdentifier).diskFileCount
+      }
+      resourceConsumptionSource.addGauge(
+        ResourceConsumptionSource.DISK_BYTES_WRITTEN,
+        userIdentifier.toMap) { () =>
+        workerInfo.userResourceConsumption.get(userIdentifier).diskBytesWritten
+      }
+      resourceConsumptionSource.addGauge(
+        ResourceConsumptionSource.HDFS_FILE_COUNT,
+        userIdentifier.toMap) { () =>
+        workerInfo.userResourceConsumption.get(userIdentifier).hdfsFileCount
+      }
+      resourceConsumptionSource.addGauge(
+        ResourceConsumptionSource.HDFS_BYTES_WRITTEN,
+        userIdentifier.toMap) { () =>
+        workerInfo.userResourceConsumption.get(userIdentifier).hdfsBytesWritten
+      }
+    }
+    workerInfo.updateThenGetUserResourceConsumption(resourceConsumptionSnapshot.asJava)
+  }
+
   @VisibleForTesting
   def cleanup(expiredShuffleKeys: JHashSet[String], threadPool: ThreadPoolExecutor): Unit =
     synchronized {
@@ -546,6 +584,15 @@ private[celeborn] class Worker(
     sb.toString()
   }
 
+  override def getApplicationList: String = {
+    val sb = new StringBuilder
+    sb.append("================= LifecycleManager Application List ======================\n")
+    workerInfo.getApplicationIdSet.asScala.foreach { appId =>
+      sb.append(s"$appId\n")
+    }
+    sb.toString()
+  }
+
   override def getShuffleList: String = {
     val sb = new StringBuilder
     sb.append("======================= Shuffle Key List ============================\n")
@@ -589,13 +636,13 @@ private[celeborn] class Worker(
     val sb = new StringBuilder
     sb.append("==================== Unavailable Peers of Worker =====================\n")
     unavailablePeers.asScala.foreach { case (peer, time) =>
-      sb.append(s"${peer.toUniqueId().padTo(50, " ").mkString}${dateFmt.format(time)}\n")
+      sb.append(s"${peer.toUniqueId().padTo(50, " ").mkString}${Utils.formatTimestamp(time)}\n")
     }
     sb.toString()
   }
 
   override def exit(exitType: String): String = {
-    exitType match {
+    exitType.toUpperCase(Locale.ROOT) match {
       case "DECOMMISSION" =>
         exitKind = CelebornExitKind.WORKER_DECOMMISSION
         ShutdownHookManager.get().updateTimeout(
