@@ -26,6 +26,12 @@ class WorkerSource(conf: CelebornConf) extends AbstractSource(conf, MetricsSyste
 
   import WorkerSource._
   // add counters
+  addCounter(OPEN_STREAM_SUCCESS_COUNT)
+  addCounter(OPEN_STREAM_FAIL_COUNT)
+  addCounter(FETCH_CHUNK_SUCCESS_COUNT)
+  addCounter(FETCH_CHUNK_FAIL_COUNT)
+  addCounter(WRITE_DATA_HARD_SPLIT_COUNT)
+  addCounter(WRITE_DATA_SUCCESS_COUNT)
   addCounter(WRITE_DATA_FAIL_COUNT)
   addCounter(REPLICATE_DATA_FAIL_COUNT)
   addCounter(REPLICATE_DATA_WRITE_FAIL_COUNT)
@@ -68,15 +74,24 @@ class WorkerSource(conf: CelebornConf) extends AbstractSource(conf, MetricsSyste
 }
 
 object WorkerSource {
-  val COMMIT_FILES_TIME = "CommitFilesTime"
-  val RESERVE_SLOTS_TIME = "ReserveSlotsTime"
-  val FLUSH_DATA_TIME = "FlushDataTime"
+  val REGISTERED_SHUFFLE_COUNT = "RegisteredShuffleCount"
+
+  val RUNNING_APPLICATION_COUNT = "RunningApplicationCount"
+
+  // fetch data
   val OPEN_STREAM_TIME = "OpenStreamTime"
   val FETCH_CHUNK_TIME = "FetchChunkTime"
+  val CHUNK_STREAM_COUNT = "ChunkStreamCount"
+  val OPEN_STREAM_SUCCESS_COUNT = "OpenStreamSuccessCount"
+  val OPEN_STREAM_FAIL_COUNT = "OpenStreamFailCount"
+  val FETCH_CHUNK_SUCCESS_COUNT = "FetchChunkSuccessCount"
+  val FETCH_CHUNK_FAIL_COUNT = "FetchChunkFailCount"
 
   // push data
   val PRIMARY_PUSH_DATA_TIME = "PrimaryPushDataTime"
   val REPLICA_PUSH_DATA_TIME = "ReplicaPushDataTime"
+  val WRITE_DATA_HARD_SPLIT_COUNT = "WriteDataHardSplitCount"
+  val WRITE_DATA_SUCCESS_COUNT = "WriteDataSuccessCount"
   val WRITE_DATA_FAIL_COUNT = "WriteDataFailCount"
   val REPLICATE_DATA_FAIL_COUNT = "ReplicateDataFailCount"
   val REPLICATE_DATA_WRITE_FAIL_COUNT = "ReplicateDataWriteFailCount"
@@ -92,15 +107,21 @@ object WorkerSource {
   val REPLICA_REGION_START_TIME = "ReplicaRegionStartTime"
   val PRIMARY_REGION_FINISH_TIME = "PrimaryRegionFinishTime"
   val REPLICA_REGION_FINISH_TIME = "ReplicaRegionFinishTime"
+
+  // pause push data
   val PAUSE_PUSH_DATA_TIME = "PausePushDataTime"
+  val PAUSE_PUSH_DATA_AND_REPLICATE_TIME = "PausePushDataAndReplicateTime"
+  val PAUSE_PUSH_DATA_COUNT = "PausePushData"
+  val PAUSE_PUSH_DATA_AND_REPLICATE_COUNT = "PausePushDataAndReplicate"
 
   // flush
   val TAKE_BUFFER_TIME = "TakeBufferTime"
-
-  val REGISTERED_SHUFFLE_COUNT = "RegisteredShuffleCount"
+  val FLUSH_DATA_TIME = "FlushDataTime"
+  val COMMIT_FILES_TIME = "CommitFilesTime"
 
   // slots
   val SLOTS_ALLOCATED = "SlotsAllocated"
+  val RESERVE_SLOTS_TIME = "ReserveSlotsTime"
 
   // connection
   val ACTIVE_CONNECTION_COUNT = "ActiveConnectionCount"
@@ -113,11 +134,11 @@ object WorkerSource {
   val SORTED_FILES = "SortedFiles"
   val SORTED_FILE_SIZE = "SortedFileSize"
   val DISK_BUFFER = "DiskBuffer"
-  val PAUSE_PUSH_DATA_COUNT = "PausePushData"
-  val PAUSE_PUSH_DATA_AND_REPLICATE_COUNT = "PausePushDataAndReplicate"
   val BUFFER_STREAM_READ_BUFFER = "BufferStreamReadBuffer"
   val READ_BUFFER_DISPATCHER_REQUESTS_LENGTH = "ReadBufferDispatcherRequestsLength"
   val READ_BUFFER_ALLOCATED_COUNT = "ReadBufferAllocatedCount"
+
+  // credit
   val CREDIT_STREAM_COUNT = "CreditStreamCount"
   val ACTIVE_MAP_PARTITION_COUNT = "ActiveMapPartitionCount"
 
@@ -132,7 +153,7 @@ object WorkerSource {
   val USER_PRODUCE_SPEED = "UserProduceSpeed"
   val WORKER_CONSUME_SPEED = "WorkerConsumeSpeed"
 
-  // active shuffle size
+  // active shuffle
   val ACTIVE_SHUFFLE_SIZE = "ActiveShuffleSize"
   val ACTIVE_SHUFFLE_FILE_COUNT = "ActiveShuffleFileCount"
 }
