@@ -32,20 +32,20 @@ import io.netty.buffer.PooledByteBufAllocator
 import org.apache.commons.io.FileUtils
 import org.apache.hadoop.fs.{FileSystem, Path}
 import org.apache.hadoop.fs.permission.FsPermission
-import org.iq80.leveldb.{DB, WriteOptions}
 
 import org.apache.celeborn.common.CelebornConf
 import org.apache.celeborn.common.exception.CelebornException
 import org.apache.celeborn.common.identity.UserIdentifier
 import org.apache.celeborn.common.internal.Logging
 import org.apache.celeborn.common.meta.{DeviceInfo, DiskInfo, DiskStatus, FileInfo, TimeWindow}
-import org.apache.celeborn.common.metrics.source.AbstractSource
+import org.apache.celeborn.common.metrics.source.{AbstractSource, ThreadPoolSource}
 import org.apache.celeborn.common.network.util.{NettyUtils, TransportConf}
-import org.apache.celeborn.common.protocol.{PartitionLocation, PartitionSplitMode, PartitionType}
+import org.apache.celeborn.common.protocol.{PartitionLocation, PartitionSplitMode, PartitionType, StorageInfo}
 import org.apache.celeborn.common.quota.ResourceConsumption
 import org.apache.celeborn.common.util.{CelebornExitKind, CelebornHadoopUtils, JavaUtils, PbSerDeUtils, ThreadUtils, Utils}
 import org.apache.celeborn.service.deploy.worker._
 import org.apache.celeborn.service.deploy.worker.memory.MemoryManager.MemoryPressureListener
+import org.apache.celeborn.service.deploy.worker.shuffledb.{DB, DBBackend, DBProvider}
 import org.apache.celeborn.service.deploy.worker.storage.StorageManager.hadoopFs
 
 final private[worker] class StorageManager(conf: CelebornConf, workerSource: AbstractSource)
@@ -72,6 +72,10 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     DeviceInfo.getDeviceAndDiskInfos(workingDirInfos, conf)
   }
   val mountPoints = new util.HashSet[String](diskInfos.keySet())
+  val hdfsDiskInfo =
+    if (conf.hasHDFSStorage)
+      Option(new DiskInfo("HDFS", Long.MaxValue, 999999, 999999, 0, StorageInfo.Type.HDFS))
+    else None
 
   def disksSnapshot(): List[DiskInfo] = {
     diskInfos.synchronized {
@@ -90,7 +94,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
         cleaners.put(
           diskInfo.mountPoint,
           ThreadUtils.newDaemonCachedThreadPool(
-            s"disk-cleaner-${diskInfo.mountPoint}",
+            s"worker-disk-${diskInfo.mountPoint}-cleaner",
             conf.workerDiskCleanThreads))
     }
     cleaners
@@ -161,6 +165,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     if (diskStatus == DiskStatus.CRITICAL_ERROR) {
       logInfo(s"Disk $mountPoint faces critical error, will remove its disk operator.")
       val operator = diskOperators.remove(mountPoint)
+      ThreadPoolSource.unregisterSource(s"worker-disk-$mountPoint-cleaner")
       if (operator != null) {
         operator.shutdown()
       }
@@ -172,7 +177,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
       diskOperators.put(
         mountPoint,
         ThreadUtils.newDaemonCachedThreadPool(
-          s"disk-cleaner-$mountPoint",
+          s"worker-disk-$mountPoint-cleaner",
           conf.workerDiskCleanThreads))
     }
   }
@@ -190,7 +195,8 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
   // shuffleKey -> (fileName -> file info)
   private val fileInfos =
     JavaUtils.newConcurrentHashMap[String, ConcurrentHashMap[String, FileInfo]]()
-  private val RECOVERY_FILE_NAME = "recovery.ldb"
+  private val RECOVERY_FILE_NAME_PREFIX = "recovery"
+  private var RECOVERY_FILE_NAME = "recovery.ldb"
   private var db: DB = null
   private var saveCommittedFileInfosExecutor: ScheduledExecutorService = _
   private val saveCommittedFileInfoBySyncMethod =
@@ -203,8 +209,10 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
   val workerGracefulShutdown = conf.workerGracefulShutdown
   if (workerGracefulShutdown) {
     try {
+      val dbBackend = DBBackend.byName(conf.workerGracefulShutdownRecoverDbBackend)
+      RECOVERY_FILE_NAME = dbBackend.fileName(RECOVERY_FILE_NAME_PREFIX)
       val recoverFile = new File(conf.workerGracefulShutdownRecoverPath, RECOVERY_FILE_NAME)
-      this.db = LevelDBProvider.initLevelDB(recoverFile, CURRENT_VERSION)
+      this.db = DBProvider.initDB(dbBackend, recoverFile, CURRENT_VERSION)
       reloadAndCleanFileInfos(this.db)
     } catch {
       case e: Exception =>
@@ -215,7 +223,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
       JavaUtils.newConcurrentHashMap[String, ConcurrentHashMap[String, FileInfo]]()
     saveCommittedFileInfosExecutor =
       ThreadUtils.newDaemonSingleThreadScheduledExecutor(
-        "StorageManager-save-committed-fileinfo-thread")
+        "worker-storage-manager-committed-fileinfo-saver")
     saveCommittedFileInfosExecutor.scheduleWithFixedDelay(
       new Runnable {
         override def run(): Unit = {
@@ -225,7 +233,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
               db.put(
                 dbShuffleKey(shuffleKey),
                 PbSerDeUtils.toPbFileInfoMap(files),
-                new WriteOptions().sync(saveCommittedFileInfoBySyncMethod))
+                saveCommittedFileInfoBySyncMethod)
             }
           }
         }
@@ -273,16 +281,16 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
   def saveAllCommittedFileInfosToDB(): Unit = {
     // save committed fileinfo to DB should be done within the time of saveCommittedFileInfoInterval
     saveCommittedFileInfosExecutor.awaitTermination(saveCommittedFileInfoInterval, MILLISECONDS)
-    // graceful shutdown might be timed out, persist all committed fileinfos to levelDB
+    // graceful shutdown might be timed out, persist all committed fileinfos to DB
     // final flush write through
     committedFileInfos.asScala.foreach { case (shuffleKey, files) =>
       try {
+        // K8s container might gone
         db.put(
           dbShuffleKey(shuffleKey),
           PbSerDeUtils.toPbFileInfoMap(files),
-          // K8s container might gone
-          new WriteOptions().sync(true))
-        logDebug(s"Update FileInfos into DB: ${shuffleKey} -> ${files}")
+          true)
+        logDebug(s"Update FileInfos into DB: $shuffleKey -> $files")
       } catch {
         case exception: Exception =>
           logError(s"Update FileInfos into DB: ${shuffleKey} failed.", exception)
@@ -359,7 +367,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
         throw new IOException(s"No available disks! suggested mountPoint $suggestedMountPoint")
       }
       val shuffleKey = Utils.makeShuffleKey(appId, shuffleId)
-      if (dirs.isEmpty && location.getStorageInfo.HDFSAvailable()) {
+      if ((dirs.isEmpty && location.getStorageInfo.HDFSAvailable()) || location.getStorageInfo.HDFSOnly()) {
         val shuffleDir =
           new Path(new Path(hdfsDir, conf.workerWorkingDir), s"$appId/$shuffleId")
         val fileInfo =
@@ -592,7 +600,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
   }
 
   private val storageScheduler =
-    ThreadUtils.newDaemonSingleThreadScheduledExecutor("storage-scheduler")
+    ThreadUtils.newDaemonSingleThreadScheduledExecutor("worker-storage-manager-scheduler")
 
   storageScheduler.scheduleWithFixedDelay(
     new Runnable {
@@ -705,7 +713,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
           db.close()
         } catch {
           case exception: Exception =>
-            logError("Store recover data to LevelDB failed.", exception)
+            logError("Store recover data to DB failed.", exception)
         }
       } else {
         if (db != null) {
@@ -757,7 +765,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     })
     hdfsWriters.forEach(new BiConsumer[String, FileWriter] {
       override def accept(t: String, u: FileWriter): Unit = {
-        u.flushOnMemoryPressure();
+        u.flushOnMemoryPressure()
       }
     })
   }
@@ -768,11 +776,12 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
 
   override def onTrim(): Unit = {
     logInfo(s"Trigger ${this.getClass.getCanonicalName} trim action")
-    flushFileWriters()
     try {
+      flushFileWriters()
       Thread.sleep(conf.workerDirectMemoryTrimFlushWaitInterval)
     } catch {
-      case _: Exception => // Do nothing
+      case e: Exception =>
+        logError(s"Trigger ${this.getClass.getCanonicalName} trim failed.", e)
     }
   }
 
